#pragma once

#include "Buffer.hpp"

#include <vector>
#include <functional>
#include <stdint.h>

namespace megu {
    template <typename T, uint32_t Y>
    class DynamicBuffer : public Buffer {
        public:
            DynamicBuffer() = delete;
            DynamicBuffer(size_t, size_t=0);
            DynamicBuffer(const DynamicBuffer &) = delete;

            virtual void generate(const std::vector<T> &) = 0;
            
            size_t size() const;
            size_t max_size() const;
            size_t memory_size() const;
            size_t max_memory_size() const;

            bool empty() const;
            bool full() const;

            void set(const std::vector<T> &);
            
            void insert(const std::vector<T> &, size_t pos);
            void replace(const std::vector<T> &, size_t pos);
            void erase(size_t from,size_t to);

            void merge(const DynamicBuffer<T,Y> &, size_t pos=0);

            void push_back(const std::vector<T> &);
            void push_front(const std::vector<T> &);

            void regenerate(size_t max_size);
            void reset(const std::vector<T> &);
            void clear();

            void resize(size_t max_size);

            void pop_back(size_t offset=1);
            void pop_front(size_t offset=1);

            T back() const;
            T front() const;

            T at(size_t index) const;

            std::vector<T> extract() const;
            std::vector<T> extract(size_t from, size_t to) const;

            T operator[](size_t index) const;

            void modify(std::function<void(void *)>);
            void operator()(std::function<void(void *)>);

            void operator<<(const DynamicBuffer<T,Y> &);

        private:
            size_t _max_size;
            size_t _size;
    };
}

#include "DynamicBuffer.tpp"