#include "ElementBuffer.hpp"

namespace megu {
    ElementBuffer::ElementBuffer(const VertexArray & vao, size_t size, EditMode mode)
    : DynamicBuffer(size), _vao(&vao), _id(0), _mode(mode) {
        this->generate({});
    }

    ElementBuffer::ElementBuffer(const VertexArray & vao, const std::vector<unsigned int> & indices, EditMode mode)
    : DynamicBuffer(indices.size(), indices.size()), _vao(&vao), _id(0), _mode(mode) {
        this->generate(indices);
    }

    ElementBuffer::~ElementBuffer() {
        this->unbind();
        glDeleteBuffers(1,&this->_id);
    }

    void ElementBuffer::bind() const {
        if(this->_id != 0) {
            this->_vao->bind();
            glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,this->_id);
        }
    }

    void ElementBuffer::unbind() const {
        GLint ebo = 0;
        glGetIntegerv(GL_ELEMENT_ARRAY_BUFFER, &ebo);
        if(static_cast<GLuint>(ebo) == this->_id) {
            this->_vao->unbind();
            glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
        }
    }

    void ElementBuffer::generate(const std::vector<unsigned int> & vertices) {
        this->_vao->bind();

        if(this->_id != 0) {
            glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
            glDeleteBuffers(1,&this->_id);
        }

        glGenBuffers(1,&this->_id);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,this->_id);
        glBufferData(GL_ELEMENT_ARRAY_BUFFER,this->max_size()*sizeof(unsigned int),vertices.size() == 0 ? NULL : vertices.data(), this->_mode);

    }

    bool ElementBuffer::operator==(const ElementBuffer & ebo) const {
        return this->_id == ebo._id;
    }

    bool ElementBuffer::operator!=(const ElementBuffer & ebo) const {
        return this->_id != ebo._id;
    }

    bool ElementBuffer::operator>(const ElementBuffer & ebo) const {
        return this->_id > ebo._id;
    }

    bool ElementBuffer::operator>=(const ElementBuffer & ebo) const {
        return this->_id >= ebo._id;
    }

    bool ElementBuffer::operator<(const ElementBuffer & ebo) const {
        return this->_id < ebo._id;
    }

    bool ElementBuffer::operator<=(const ElementBuffer & ebo) const {
        return this->_id <= ebo._id;
    }

    bool ElementBuffer::operator()(const ElementBuffer & ebo) const {
        return *this < ebo;
    }

    std::strong_ordering ElementBuffer::operator<=>(const ElementBuffer & ebo) const {
        return this->_id <=> ebo._id; 
    }

    ElementBuffer & operator<<(ElementBuffer & ebo, const std::initializer_list<unsigned int> & data) {
        ebo.push_back(data);
        return ebo;
    }

    ElementBuffer & operator<<(ElementBuffer & ebo, const std::vector<unsigned int> & data) {
        ebo.push_back(data);
        return ebo;
    }
}