#pragma once

#include "DynamicBuffer.hpp"

#include <compare>
#include "VertexArray.hpp"
#include "EditMode.hpp"

namespace megu {
    class ElementBuffer : public DynamicBuffer<unsigned int, GL_ELEMENT_ARRAY_BUFFER> {
        public:
            ElementBuffer() = delete;
            ElementBuffer(const VertexArray & vao, size_t size, EditMode mode = EditMode::DYNAMIC);
            ElementBuffer(const VertexArray & vao, const std::vector<unsigned int> & indice, EditMode mode = EditMode::DYNAMIC);
            ElementBuffer(const ElementBuffer &) = delete;
   virtual ~ElementBuffer();

            virtual inline GLuint identifier() const noexcept {return this->_id;}

            virtual void bind() const;
            virtual void unbind() const;

            virtual void generate(const std::vector<unsigned int> &);

            inline bool operator==(const ElementBuffer &) const;
            inline bool operator!=(const ElementBuffer &) const;
            inline bool operator<=(const ElementBuffer &) const;
            inline bool operator>=(const ElementBuffer &) const;
            inline bool operator()(const ElementBuffer &) const;

            inline bool operator<(const ElementBuffer &) const;
            inline bool operator>(const ElementBuffer &) const;

            inline std::strong_ordering operator<=>(const ElementBuffer &) const;

            friend ElementBuffer & operator<<(ElementBuffer &, const std::initializer_list<unsigned int> &);
            friend ElementBuffer & operator<<(ElementBuffer &, const std::vector<unsigned int> &);

        private:
            const VertexArray * _vao;
            GLuint _id;
            GLuint _mode;
    };
}