#include "ShaderStorageBuffer.hpp"

namespace megu {
    template <class T>
    ShaderStorageBuffer<T>::ShaderStorageBuffer(T * data) 
    : _data(data), _id(0) {
        glCreateBuffers(1, &this->_id);
        glBindBuffer(GL_SHADER_STORAGE_BUFFER, this->_id);
        glBufferData(GL_SHADER_STORAGE_BUFFER, sizeof(T), data, GL_DYNAMIC_DRAW);
    }

    template <class T>
    ShaderStorageBuffer<T>::~ShaderStorageBuffer() {
        glDeleteBuffers(1, &this->_id);
    }

    template <class T>
    void ShaderStorageBuffer<T>::setData(T * data) {
        this->_data = data;
        glBindBuffer(GL_SHADER_STORAGE_BUFFER, this->_id);
        glBufferSubData(GL_SHADER_STORAGE_BUFFER, 0, sizeof(T), data);
    }

    template <class T>
    void ShaderStorageBuffer<T>::attach(const std::string & name, const Program & program) {
        this->_programs.insert(std::pair<const Program *, GLuint>(&program, 0));
        glBindBuffer(GL_SHADER_STORAGE_BUFFER, this->_id);
        this->_programs[&program] = glGetProgramResourceIndex(program.id(), GL_SHADER_STORAGE_BLOCK, name.c_str());
        glBindBufferBase(GL_SHADER_STORAGE_BUFFER, this->_programs[&program], this->_id);
    }

    template <class T>
    void ShaderStorageBuffer<T>::bind() const {
        glBindBuffer(GL_SHADER_STORAGE_BUFFER, this->_id);
    }

    template <class T>
    void ShaderStorageBuffer<T>::unbind() const {
        GLint binded = 0;
        glGetIntegerv(GL_SHADER_STORAGE_BUFFER, &binded);
        if(static_cast<GLuint>(binded) == this->_id) {
            glBindBuffer(GL_SHADER_STORAGE_BUFFER, 0);
        }
    }
}