#pragma once

#include "Buffer.hpp"

#include <optional>
#include <string>
#include <map>

#include "../shaders/Program.hpp"

namespace megu {
    template <class T>
    class UniformeBuffer : public Buffer {
        public:
            UniformeBuffer(T * = nullptr);
   virtual ~UniformeBuffer();

            inline GLuint index(const Program & program) const {return this->_programs.at(&program);} 
            inline std::optional<T> data() const {this->_data != nullptr ? std::optional<T>(*this->_data) : std::optional<T>();}
            
            void setData(T *);
            void attach(const std::string &, const Program &);

            inline virtual GLuint identifier() const noexcept {return this->_id;}

            virtual void bind() const;
            virtual void unbind() const;
    
        private:
            T * _data;
            GLuint _id;
            std::map<const Program *, GLuint> _programs;

    }; 
}

#include "UniformeBuffer.tpp"