#include "UniformeBuffer.hpp"

namespace megu {
    template <class T>
    UniformeBuffer<T>::UniformeBuffer(T * data)
    : _data(data), _id(0) {
        glCreateBuffers(1, &this->_id);
        glBindBuffer(GL_UNIFORM_BUFFER, this->_id);
        glBufferData(GL_UNIFORM_BUFFER, sizeof(T), data, GL_DYNAMIC_DRAW);
    }

    template <class T>
    UniformeBuffer<T>::~UniformeBuffer() {
        glDeleteBuffers(1, &this->_id);
    }

    template <class T>
    void UniformeBuffer<T>::setData(T * data) {
        this->_data = data;
        glBindBuffer(GL_UNIFORM_BUFFER, this->_id);
        glBufferSubData(GL_UNIFORM_BUFFER, 0, sizeof(T), data);
    }

    template <class T>
    void UniformeBuffer<T>::attach(const std::string & name, const Program & program) {
        this->_programs.insert(std::pair<const Program *, GLuint>(&program, 0));
        glBindBuffer(GL_UNIFORM_BUFFER, this->_id);
        this->_programs[&program] = glGetUniformBlockIndex(program.id(), name.c_str());
		glBindBufferBase(GL_UNIFORM_BUFFER, this->_programs[&program], this->_id);
    }

    template <class T>
    void UniformeBuffer<T>::bind() const {
        glBindBuffer(GL_UNIFORM_BUFFER, this->_id);
    }

    template <class T>
    void UniformeBuffer<T>::unbind() const {
        GLint binded = 0;
        glGetIntegerv(GL_UNIFORM_BUFFER_BINDING, &binded);
        if(static_cast<GLuint>(binded) == this->_id) {
            glBindBuffer(GL_UNIFORM_BUFFER, 0);
        }
    }
}