#include "VertexArray.hpp"

namespace megu {
    VertexArray::VertexArray(bool binded) 
    : _vao(0) {
        glGenVertexArrays(1, &this->_vao);

        if(binded) {
            this->bind();
        }
    }

    VertexArray::~VertexArray() {
        this->unbind();
        glDeleteVertexArrays(1,&this->_vao);
    }

    void VertexArray::bind() const {
        if(this->_vao != 0) {
            glBindVertexArray(this->_vao);
        }
    }

    void VertexArray::unbind()const {
        GLint vao;
        glGetIntegerv(GL_ARRAY_BUFFER, &vao);
        if(static_cast<GLuint>(vao) == this->_vao) {
            glBindVertexArray(0);
        }
    }

    bool VertexArray::operator==(const VertexArray & vao) const {
        return this->_vao == vao._vao;
    }

    bool VertexArray::operator!=(const VertexArray & vao) const {
        return this->_vao != vao._vao;
    }

    bool VertexArray::operator>(const VertexArray & vao) const {
        return this->_vao > vao._vao;
    }

    bool VertexArray::operator>=(const VertexArray & vao) const {
        return this->_vao >= vao._vao;
    }

    bool VertexArray::operator()(const VertexArray & vao) const {
        return *this > vao;
    }

    bool VertexArray::operator<(const VertexArray & vao) const {
        return this->_vao < vao._vao;
    }

    bool VertexArray::operator<=(const VertexArray & vao) const {
        return this->_vao <= vao._vao;
    }

    std::strong_ordering VertexArray::operator<=>(const VertexArray & vao) const {
        return this->_vao <=> vao._vao; 
    }
}