#pragma once

#include "DynamicBuffer.hpp"

#include <compare>

#include "VertexArray.hpp"
#include "Layout.hpp"
#include "EditMode.hpp"

namespace megu {
    class VerticeBuffer : public DynamicBuffer<float, GL_ARRAY_BUFFER> {
        public:
            VerticeBuffer() = delete;
            VerticeBuffer(const VertexArray & vao, const Layout_t &, size_t, EditMode=EditMode::DYNAMIC);
            VerticeBuffer(const VertexArray & vao, const Layout_t &, const std::vector<float> &, EditMode=EditMode::DYNAMIC);
            VerticeBuffer(const VerticeBuffer &) = delete;
   virtual ~VerticeBuffer();

            const Layout_t & layout() const {return this->_layout;}

            size_t weight() const;
            size_t vertices() const;

            virtual GLuint identifier() const noexcept {return this->_id;}
           
            virtual void bind() const;
            virtual void unbind() const;

            virtual void generate(const std::vector<float> &);

            bool compare(const Layout_t &) const;

            bool operator==(const VerticeBuffer &) const;
            bool operator!=(const VerticeBuffer &) const;
            bool operator<=(const VerticeBuffer &) const;
            bool operator>=(const VerticeBuffer &) const;
            bool operator()(const VerticeBuffer &) const;
            
            bool operator<(const VerticeBuffer &) const;
            bool operator>(const VerticeBuffer &) const;

            std::strong_ordering operator<=>(const VerticeBuffer &) const;

            friend VerticeBuffer & operator<<(VerticeBuffer &, const std::initializer_list<float> &);
            friend VerticeBuffer & operator<<(VerticeBuffer &, const std::vector<float> &);

        private:
            const VertexArray * _vao;
            GLuint _id;
            EditMode _mode;
            Layout_t _layout;
    };

    
}