#include "Target.hpp"

#include <glm/gtc/matrix_transform.hpp>

#include <iostream>

namespace megu {
    Target::Target(const glm::vec3 & position, const glm::vec3 & target, double fov, double near, double far)
    : _position(position), _target(target), _rotation(0.f, 0.f, 0.f), _view(1.f), _projection(1.f), _fov(fov), _ratio(1.f), _near(near), _far(far), _update(0) {
        this->update(Update_Flags::VIEW | Update_Flags::PROJECTION);
    }

    void Target::update(uint8_t flags) const {
        this->_update |= flags;
        if(this->_update != Update_Flags::NONE) {

            if((this->_update & Update_Flags::VIEW) != 0) {
                this->_view = glm::lookAt(this->_position, this->_target, glm::vec3(0.0f, 1.0f, 0.0f));
                this->_view = glm::rotate(this->_view, glm::radians(this->_rotation.x), {1.f, 0.f, 0.f});
                this->_view = glm::rotate(this->_view, glm::radians(this->_rotation.y), {0.f, 1.f, 0.f});
                this->_view = glm::rotate(this->_view, glm::radians(this->_rotation.z), {0.f, 0.f, 1.f});
            }

            if((this->_update & Update_Flags::PROJECTION) != 0) {
                this->_projection = glm::perspective(glm::radians(this->_fov), this->_ratio, this->_near, this->_far);
            }   

            this->_update = Update_Flags::NONE;
        }
    }

    void Target::setPosition(double x, double y, double z) {
        this->_position = {x, y, z};
        this->_update |= VIEW;
    }

    void Target::setTarget(double x, double y, double z) {
        this->_target = {x, y, z};
        this->_update |= VIEW;
    }

    void Target::setRotation(double x, double y, double z) {
        this->_rotation = {x, y, z};
        this->_update |= VIEW;
    }

    void Target::setFov(double fov) {
        this->_fov = fov;
        this->_update |= PROJECTION;
    }

    void Target::setRatio(double ratio) {
        this->_ratio = ratio;
        this->_update |= PROJECTION;
    }

    void Target::setViewDistance(double near, double far) {
        this->_near = near;
        this->_far = far;
        this->_update |= PROJECTION;
    }

    void Target::move(const glm::vec3 & movement) {
        this->_position += movement;
        this->_update |= VIEW; 
    }

    void Target::rotate(const glm::vec3 & rotation) {
        this->_rotation += rotation;
        this->_view = glm::rotate(this->_view, glm::radians(rotation.x), {1.f, 0.f, 0.f});
        this->_view = glm::rotate(this->_view, glm::radians(rotation.y), {0.f, 1.f, 0.f});
        this->_view = glm::rotate(this->_view, glm::radians(rotation.z), {0.f, 0.f, 1.f});
    }

    const glm::mat4 & Target::view() const {
        this->update();
        return this->_view;
    }

    const glm::mat4 & Target::projection() const {
        this->update();
        return this->_projection;
    }
}