#pragma once

#include "Camera.hpp"

#include <glm/vec3.hpp>

namespace megu {
    class Target : public Camera {
        private:
            glm::vec3 _position;
            glm::vec3 _target;
            glm::vec3 _rotation;

            mutable glm::mat4 _view;
            mutable glm::mat4 _projection;

            double _fov;
            double _ratio;
            double _near;
            double _far;

            mutable uint8_t _update;

            void update(uint8_t flags = NONE) const;

        public:
            Target(const glm::vec3 & = {0.f, 0.f, 3.f}, const glm::vec3 & = {0.f, 0.f, 0.f}, double=65.f, double=0.1f, double=100.f);

            inline const glm::vec3 & getPosition() const {return this->_position;}
            inline const glm::vec3 & getTarget() const {return this->_target;}

            inline double getFov() const {return this->_fov;}
            inline double getRatio() const {return this->_ratio;}
            inline double getNear() const {return this->_near;}
            inline double getFar() const {return this->_far;}

            void setPosition(double, double, double);
            void setTarget(double, double, double);
            void setRotation(double, double, double);
            
            void setFov(double fov);
            void setRatio(double ratio);
            void setViewDistance(double near, double far); 

            void move(const glm::vec3 & movement);
            void rotate(const glm::vec3 & rotation);

            inline void move(double x, double y, double z = 0.0) {this->move({x, y, z});}
            inline void rotate(double pitch, double yaw, double roll = 0.0) {this->rotate({pitch, yaw, roll});}
 
            const glm::mat4 & view() const;
            const glm::mat4 & projection() const;
    };
}