#include "View.hpp"

#include <glm/gtc/matrix_transform.hpp>

namespace megu {
    View::View(float x, float y, float w, float h, float near, float far) 
    : View({x, y}, {w, h}, near, far) {}

    View::View(const glm::vec2 & position, const glm::vec2 & dimension, float near, float far)
    : _view(1.f), _projection(1.f), _position(position), _dimension(dimension), _roll(0.f), _near(near), _far(far), _update() {
        this->update(Update_Flags::VIEW | Update_Flags::PROJECTION);
    }

    void View::update(uint8_t flags) const {
        this->_update |= flags;
        if(this->_update != Update_Flags::NONE) {
            if((this->_update & VIEW) != 0) {
                this->_view = glm::rotate(this->_view, this->_roll, {0.0, 0.0, 1.0});
            }

            if((this->_update & PROJECTION) != 0) {
                glm::vec2 center = {this->_position.x + this->_dimension.x / 2, this->_position.y + this->_dimension.y / 2};
                this->_projection = glm::ortho(center.x - this->_dimension.x/2, center.x + this->_dimension.x/2, center.y - this->_dimension.y/2, center.y + this->_dimension.y/2, this->_near, this->_far);
            }
        }
        this->_update = Update_Flags::NONE;
    }

    void View::setPosition(const glm::vec2 & position) {
        this->_position = position;
        this->_update |= Update_Flags::PROJECTION;
    }

    void View::setDimension(const glm::vec2 & dimension) {
        this->_dimension = dimension;
        this->_update |= Update_Flags::PROJECTION;
    }

    void View::setDistance(float near, float far) {
        this->_near = near;
        this->_far = far;
        this->_update |= Update_Flags::PROJECTION;
    }

    void View::move(const glm::vec2 & position) {
        this->_position += position;
        this->_update |= Update_Flags::PROJECTION;
    }

    void View::setRoll(float roll) {
        this->_roll = roll;
        this->_update |= Update_Flags::VIEW;
    }

    void View::roll(float roll) {
        this->_roll += roll;
        this->_update |= Update_Flags::VIEW;
    } 

    const glm::mat4 & View::view() const {
        this->update();
        return this->_view;
    }

    const glm::mat4 & View::projection() const {
        this->update();
        return this->_projection;
    }
}