#pragma once

#include "Camera.hpp"

#include <glm/vec2.hpp>
#include <algorithm>
#include <iterator>
#include <set>

namespace megu {
    class View : public Camera {
        private:
            mutable glm::mat4 _view;
            mutable glm::mat4 _projection;
            glm::vec2 _position;
            glm::vec2 _dimension;
            
            float _roll;
            float _near;
            float _far;

            mutable uint8_t _update;
            
            void update(uint8_t = NONE) const;

        public:
            View() = delete;
            View(float x, float y, float w, float h, float = -10, float = 10.0);
            View(const glm::vec2 &, const glm::vec2 &, float = -10, float = 10.0);

            inline const glm::vec2 & position() const {return this->_position;}
            inline const glm::vec2 & dimension() const {return this->_dimension;}
            
            inline float roll() const {return this->_roll;}
            inline float getNear() const {return this->_near;}
            inline float getFar() const {return this->_far;}

            void setPosition(const glm::vec2 &);
            void setDimension(const glm::vec2 &);
            void setRoll(float);
            void setDistance(float near, float far);

            void move(const glm::vec2 &);
            void roll(float);

            inline void move(float x, float y) {this->move({x, y});}

            const glm::mat4 & view() const;
            const glm::mat4 & projection() const;
    };
}