#pragma once

#include "Camera.hpp"

#include <glm/vec3.hpp>

namespace megu {
    class Walkaround : public Camera {
        private:
            mutable glm::mat4 _view;
            mutable glm::mat4 _projection;

            glm::vec3 _position;
            mutable glm::vec3 _up;

            double _pitch;
            double _yaw;
            double _roll;

            double _fov;
            double _ratio;
            double _near;
            double _far;

            mutable uint8_t _update;

            void update(uint8_t = NONE) const;

        public:
            Walkaround();
            Walkaround(const glm::vec3 &, double = 70.0f, double = 1.f, double = 0.1f, double = 100.0f);

            inline const glm::vec3 & position() const {return this->_position;}
            
            inline double pitch() const {return this->_pitch;}
            inline double roll() const {return this->_roll;}
            inline double yaw() const {return this->_yaw;}

            inline double ratio() const {return this->_ratio;}
            inline double getNear() const {return this->_near;}
            inline double getFar() const {return this->_far;}
            inline double fov() const {return this->_fov;}

            void setPosition(const glm::vec3 &);
            void setPitch(double);
            void setYaw(double);
            void setRoll(double);
            void setRatio(double);
            void setDistance(double near, double far);

            void move(const glm::vec3 &);
            void rotate(const glm::vec3 &);

            inline void move(double x, double y, double z = 0.0) {this->move({x, y, z});}
            inline void rotate(double pitch, double yaw, double roll = 0.0) {this->rotate({pitch, yaw, roll});}

            inline void setRatio(double w, double h) {this->setRatio(w/h);}

            const glm::mat4 & view() const;
            const glm::mat4 & projection() const;
            
    };
}