#pragma once

#include <list>
#include <typeindex>

namespace megu {
    template <typename T>
    class Counter {
        public:
            template <T U>
            static std::list<T> Count() {
                std::list<T> list;
                list.push_back(U);
                return list;
            }

            template <T U>
            static std::list<T> Count(std::list<T> & list) {
                list.push_back(U);
                return list;
            }

            template <T U, T V, T ... L>
            static std::list<T> Count() {
                std::list<T> list;
                list.push_back(U);
                return Counter<T>::Count<V, L...>(list);
            }

            template <T U, T V, T ... L>
            static std::list<T> Count(std::list<T> & list) {
                list.push_back(U);
                return Counter<T>::Count<V, L...>(list);
            }
    };

    class Type_Counter {
        public:
            template <class T>
            static std::list<std::type_index> Count() {
                std::list<std::type_index> list;
                list.push_back(typeid(T));
                return list;
            }

            template <class T>
            static std::list<std::type_index> Count(std::list<std::type_index> & list) {
                list.push_back(typeid(T));
                return list;
            }

            template <class T, class U, class ... V>
            static std::list<std::type_index> Count() {
                std::list<std::type_index> list;
                list.push_back(typeid(T));
                return Type_Counter::Count<U, V...>(list);
            }

            template <class T, class U, class ... V>
            static std::list<std::type_index> Count(std::list<std::type_index> & list) {
                list.push_back(typeid(T));
                return Type_Counter::Count<U, V...>(list);
            }
    };

    template <typename T>
    class Sum {
        public:
            template <T U>
            static constexpr size_t sum() {
                size_t sum = U;
                return sum;
            }   

            template <T U>
            static constexpr size_t sum(size_t & sum) {
                sum += U;
                return sum;
            }

            template <T U, T V, T ... S>
            static constexpr size_t sum() {
                size_t sum = U;
                return Sum::sum<V, S...>(sum);
            }

            template <T U, T V, T ... S>
            static constexpr size_t sum(size_t & sum) {
                sum += U;
                return Sum::sum<V, S...>(sum);
            }
    };
}