#pragma once

#include <stdint.h>
#include <vector>
#include <array>
#include <thread>

#include <GL/glew.h>

#include "../buffers/Layout.hpp"
#include "Counter.hpp"

namespace megu {
    using Vertices_t = std::vector<float>;
    using Elements_t = std::vector<unsigned int>;

    enum Primitive_t {
        TRIANGLES   = GL_TRIANGLES,
        QUADS       = GL_QUADS,
    };

    template <Primitive_t P>
    class Primitived {
        public: 
            inline static Primitive_t Primitive() {return P;}
    };

    template <Primitive_t P>
    class Geometry : public Primitived<P> {
        public: 
            Geometry() = delete;
            Geometry(const Layout_t &);
           ~Geometry() = default;

            virtual const Vertices_t & vertices() const = 0;

            inline const Layout_t & layout() const {return this->_layout;}
            
        private:
            Layout_t _layout;
    };

    template <Primitive_t P>
    class Geometry_Indiced : public Geometry<P> {
        public:
            Geometry_Indiced() = delete;
            Geometry_Indiced(const Layout_t &);
           ~Geometry_Indiced() = default;

            virtual const Elements_t & elements() const = 0;
    };

    template <class T, Primitive_t P, layout::Value ... L>
    class Static_Geometry : public Primitived<P> {
        public:
            Static_Geometry() = default;
           ~Static_Geometry() = default;

            const Vertices_t & vertices() const {return Static_Geometry<T, P>::Vertices();}

            static Layout_t Layout() {return Counter<megu::layout::Value>::Count<L...>();}
            static const Vertices_t & Vertices()  {return T::Vertices();}
            static const size_t & Count() {return T::Count();}
    };

    template <class T, Primitive_t P, layout::Value ... L>
    class Static_Geometry_Indiced : public Geometry_Indiced<P>, public Static_Geometry<T, P, L...> {
        public:
            Static_Geometry_Indiced() = default;
           ~Static_Geometry_Indiced() = default;

            const Elements_t  & elements() const final override {return Static_Geometry_Indiced<T, P>::Elements();}
            static const Elements_t & Elements() {return T::Elements();}
    };   
}

#include "Geometry.tpp"