#pragma once

#include <GL/glew.h>
#include <glm/glm.hpp>
#include <vector>
#include <set>
#include <unordered_map>

#include "Source.hpp"

namespace megu {
    class Program {
        private:
            GLuint _id;
            bool _usable;
            uint8_t _attached;
            mutable std::unordered_map<std::string, GLuint> _locations;

        protected:
            enum Bit_Categorie {
                NONE            = 0,
                VERTEX          = 1,
                FRAGMENT        = 2,
                GEOMETRY        = 4,
                TESS_CONTROL    = 8,
                TESS_EVALUATION = 16,
                COMPUTE         = 32,
            }; 

            static const uint8_t Categorie(const Source::Categorie);

        public:
            Program();
            Program(const Program &) = delete;
            Program(const Program &&) = delete;
            Program operator=(const Program &) = delete;
   virtual ~Program();

            void attach(const Source &);
            void dettach(const Source &);
            void link();
            void kill();

            virtual void use() const;

            inline GLuint id() const {return this->_id;}
            inline bool usable() const {return this->_usable;}

            inline void operator<<(const Source & source) {this->attach(source);}

            void setUniform(const std::string &, GLint) const;
            void setUniform(const std::string &, GLuint) const;
            void setUniform(const std::string &, GLfloat) const;
            void setUniform(const std::string &, GLdouble) const;
            void setUniform(const std::string &, GLchar) const;

            void setUniform(const std::string &, const glm::vec2 &) const;
            void setUniform(const std::string &, const glm::vec3 &) const;
            void setUniform(const std::string &, const glm::vec<3, uint32_t> &) const;
            void setUniform(const std::string &, const glm::vec<3, int32_t> &)  const;
            void setUniform(const std::string &, const glm::vec4 &) const;

            void setUniform(const std::string &, const glm::mat4 &) const;

            void setUniform(const std::string &, const std::vector<GLint> &) const;
            void setUniform(const std::string &, const std::vector<GLuint> &) const;
            void setUniform(const std::string &, const std::vector<GLfloat> &) const;
            void setUniform(const std::string &, const std::vector<glm::vec2> &) const;
            void setUniform(const std::string &, const std::vector<glm::vec4> &) const;
            void setUniform(const std::string &, const std::vector<glm::mat4> &) const;

            void setUniform(const std::string &, const std::vector<std::reference_wrapper<const glm::mat4>> &) const;
            void setUniform(const std::string &, const std::set<std::reference_wrapper<const glm::mat4>> &) const;

            void setUniform(const std::string &, const void *, size_t, GLenum = GL_FLOAT) const;
    };
}