#include "Source.hpp"

#include <filesystem>
#include <fstream>
#include <sstream>
#include <exception>

#include "../../errors/Shader_Error.hpp"

namespace megu {
    Source::Source() 
    : _id(0), _categorie(Categorie::NONE), _path("") {

    }

    Source::Source(const std::filesystem::path & path, Categorie categorie) 
    : _id(0), _categorie(categorie), _path("") {
        this->load(path, categorie);
    }

    Source::~Source() {
        glDeleteShader(this->_id);
    }

    void Source::load(const std::filesystem::path & path, Categorie categorie) {
        if(this->_id != 0) {
            this->release();
        }

        if(!std::filesystem::exists(path)) {
            throw std::runtime_error(std::string("Can't open : ").append(path.string()));
        }

       
        this->_path = path;
        std::ifstream is(path);

        if(is.fail()) {
            throw std::runtime_error("Stream fail.");
        }

        std::stringstream buffer;
        buffer << is.rdbuf();

        std::string ssource = buffer.str();
        const char * ccsource = ssource.c_str();

        this->_id = glCreateShader(static_cast<GLenum>(categorie));
        this->_categorie = categorie;


        glShaderSource(this->_id,1,&ccsource,NULL);
        glCompileShader(this->_id);
        buffer.clear();
        
        int success;
        char infoLog[512];
        glGetShaderiv(this->_id, GL_COMPILE_STATUS, &success);
        if(!success) {
            glGetShaderInfoLog(this->_id, 512, NULL, infoLog);
            throw error::shader_error(infoLog, &this->_path);
        }
    }

    void Source::release() {
        glDeleteShader(this->_id);
        this->_id = NO_SOURCE_ID;
        this->_categorie = Categorie::NONE;
    }
}