#pragma once

#include <GL/glew.h>
#include <filesystem>

namespace megu {
    class Source {
        public:
            enum class Categorie : GLenum {
                NONE            = 0,
                VERTEX          = GL_VERTEX_SHADER,
                FRAGMENT        = GL_FRAGMENT_SHADER,
                GEOMETRY        = GL_GEOMETRY_SHADER,
                TESS_CONTROL    = GL_TESS_CONTROL_SHADER,
                TESS_EVALUATION = GL_TESS_EVALUATION_SHADER,
                COMPUTE         = GL_COMPUTE_SHADER,
            };

        private:
            GLuint _id;
            Categorie _categorie;
            std::filesystem::path _path;

        protected:
            static constexpr GLuint NO_SOURCE_ID = 0;

        public:
            Source();
            Source(const std::filesystem::path &, Categorie categorie);
            Source(const Source &) = delete;
            Source(const Source &&) = delete;
            Source operator=(const Source &) = delete;
           ~Source();

            void load(const std::filesystem::path &, Categorie categorie);
            void release();

            inline GLuint id() const {return this->_id;}
            inline Categorie categorie() const {return this->_categorie;}

            inline bool usable() const {return this->_id != 0;}
            inline const std::filesystem::path & path() const {return this->_path;}
    };
}