#pragma once

#include <optional>
#include <variant>
#include <map>
#include <string>

#include "Program.hpp"

namespace megu {
    template <class... Ts>
    struct overloaded : Ts ... {
        using Ts::operator()...;
    };

    template <class... Ts> overloaded(Ts...) -> overloaded<Ts...>;

    template <typename ... T>
    class UniformObject {
        private:
            Program * _program;
            std::map<std::string, std::variant<T ...>> _values; 

        public:
            UniformObject() : _program(nullptr) {}
            UniformObject(Program & p) : _program(&p) {}
           ~UniformObject() {}

            template <typename U>
            void add(const std::string & name, U data) {
                this->_values.insert(std::pair<std::string, std::variant<T ...>>(name, std::variant<T ...>(data)));
            }

            template <typename U>
            void set(const std::string & name, U data) {
                this->_values.insert_or_assign(name, std::variant<T ...>(data));
            }

            template <typename U>
            U get(const std::string & name) const {
                return std::get<U>(this->_values.at(name));
            }

            bool contain(const std::string & name) const {
                return this->_values.contains(name);
            }

            inline void apply() {
                if(this->_program != nullptr) {
                    this->apply(*this->_program);
                }
            }

            void apply(const Program & prog) {
                for(auto & value : this->_values) {
                    const std::string name = value.first; 

                    std::visit(overloaded{
                        [&prog, &name](GLint & data) {       // Avoid error when passing GLint for Sample2D. 
                            prog.setUniform(name, data);
                        },
                        [&prog, &name](auto & data){ 
                            prog.setUniform(name, data);
                        }
                    }, value.second);          
                }
            } 
    };
}