#pragma once

#include <GL/glew.h>
#include <compare>

#include "TextureBuffer.hpp"

namespace megu {
    class Texture {
        public:
            Texture(GLuint = 0, GLuint = GL_TEXTURE_2D);
            Texture(const std::filesystem::path &, GLuint = 0);
            Texture(const Texture &);
            Texture & operator=(const Texture &);
           ~Texture();

            enum Wraping {
                REPEAT              = GL_REPEAT,
                MIRRORED_REPEAT     = GL_MIRRORED_REPEAT,
                CLAMP_TO_BORDER     = GL_CLAMP_TO_BORDER,
                CLAMP_TO_EDGE       = GL_CLAMP_TO_EDGE
            };

            enum Axis {
                S = 1,
                T = 2,
                R = 4
            };

            enum Format {
                RGB     = GL_RGB,
                BGR     = GL_BGR,
                RGBA    = GL_RGBA,
                RED     = GL_RED,
                GREEN   = GL_GREEN,
                BLUE    = GL_BLUE,
            };

            void bind() const;
            void bind(GLuint) const;

            inline GLint width() const {return this->_width;}
            inline GLint height() const {return this->_height;}

            inline GLuint slot() const {return this->_slot;}
            inline GLuint type() const {return this->_type;}
            inline GLuint identifier() const {return this->_id;}

            void changeSlot(GLuint);
            void setWraping(Wraping, uint32_t axis = S | T);
            void setSmoothing(bool);

            void store(GLsizei, GLsizei, const GLubyte *, uint32_t = RGBA);
            void store(const TextureBuffer &);
            bool valid() const;

            bool operator==(const Texture &) const;
            //bool operator!=(const Texture &) const;
            bool operator>=(const Texture &) const;
            bool operator<=(const Texture &) const;
            bool operator>(const Texture &) const;
            bool operator<(const Texture &) const;

            std::strong_ordering operator<=>(const Texture &) const;

            static void Unbind(GLuint = GL_TEXTURE_2D);
            static GLint CountSlots();
            static void SetDisabelingByteAligment(bool);


        private:
            GLuint _id;
            GLuint _type;
            mutable GLuint _slot;   
            GLsizei _width, _height; 
    };
}