#pragma once

#include <vector>
#include <filesystem>
#include <GL/glew.h>

namespace megu {
    class TextureBuffer {
        public:
            TextureBuffer();
            TextureBuffer(GLuint, GLuint, GLubyte *, GLenum);
            TextureBuffer(const std::filesystem::path &, bool = true);
            TextureBuffer(const TextureBuffer &);
            TextureBuffer & operator=(const TextureBuffer & src);
           ~TextureBuffer() = default;

            inline GLuint width()  const {return this->_width;}
            inline GLuint height() const {return this->_height;}
            inline GLenum format() const {return this->_format;}
            inline const std::vector<GLubyte> & data() const {return this->_data;}
            
            inline const GLubyte * data_ptr() const {return this->_data.data();}
            inline bool empty() const {return this->_data.empty();}

            void flipVerticaly();

            void load(const GLubyte *, size_t);
            void load(const std::filesystem::path &, bool = false);
            void free();

            static int BytePerPixel(GLenum);

        private:
            GLuint _width, _height;
            GLenum _format;
            std::vector<GLubyte> _data;
    };
}