#pragma once

#include <map>
#include <optional>

#include <engine/graphics/back/cameras/View.hpp>

#include "Layer.hpp"
#include "FrameBufferMerger.hpp"

namespace megu {
    class GraphicEngine {
        public:
            GraphicEngine() = delete;
            GraphicEngine(Window &, float, float);
            GraphicEngine(Window &);
           ~GraphicEngine() = default;

            void push(Priority, Renderer &);

            template <class T>
            size_t push(Priority layer_priority, Priority object_priority, T & object, Module<T> * modul) {
                return this->_layers[layer_priority].get()->push<T>(object_priority, object, modul);
            }

            void remove(Priority);
            void remove(const Identifiable &);

            std::optional<std::reference_wrapper<const Layer>> get(Priority) const;
            std::optional<std::reference_wrapper<const Renderable>> get(const Identifiable &) const;

            inline bool empty() const {return this->_layers.empty();}
            inline const std::map<Priority, std::unique_ptr<Layer>> & layers() const {return this->_layers;}
    
            void step();
            void step(Priority);
            void setClearColor(float, float, float);

        private:
            std::map<Priority, std::unique_ptr<Layer>> _layers;
            FrameBufferMerger _merger;
            Window & _window;
    };
}