#include "FrameBufferMerger.hpp"

#include <engine/graphics/front/geometry/Plane.hpp>
#include <engine/utility/array_generator.hpp>

namespace megu {
    FrameBufferMerger::FrameBufferMerger()
    : _vbo(this->_vao, Plane::Layout(), Plane::Vertices(), megu::EditMode::STATIC) {
        megu::Source vert("assets/shaders/FrameBuffer-Instanced.vert", Source::Categorie::VERTEX);
        this->_program.attach(vert);

        megu::Source frag("assets/shaders/FrameBuffer-Instanced.frag", Source::Categorie::FRAGMENT);
        this->_program.attach(frag);

        this->_program.link();

        vert.release();
        frag.release();
    }

    void FrameBufferMerger::render(const Window & window, const TextureArray & textures) {
        glViewport(0, 0, static_cast<GLsizei>(window.width()), static_cast<GLsizei>(window.height()));
        glClear(GL_COLOR_BUFFER_BIT);

        this->_program.use();
        this->_vao.bind();

        for(GLuint i = 0; i < static_cast<GLuint>(textures.size()); ++i) {
            textures[i].get().bind(static_cast<GLuint>(i));
        }   

        static auto uSampler = array_generator::Generate<GLint, 0, 32>();

        this->_program.setUniform("uSampler", uSampler);
        this->_program.setUniform("uTexturesCount", static_cast<GLuint>(textures.size()));
        glDrawArrays(Plane::Primitive(), 0, static_cast<GLsizei>(Plane::Vertices().size()));
    }
}