#pragma once

#include <map>
#include <optional>
#include <typeindex>
#include <any>

#include <engine/graphics/back/buffers/FrameBuffer.hpp>
#include <engine/graphics/front/object/Renderable.hpp>
#include <engine/utility/reference_sorter.hpp>

#include <engine/graphics/front/object/Renderable.hpp>
#include <engine/graphics/front/module/Module.hpp>

#include <engine/utility/Priority.hpp> 
#include "Renderer.hpp"

namespace megu {
    class Layer {
        public:
            Layer() = delete;
            Layer(Renderer &);
           ~Layer() = default;

            
            inline const Renderer & renderer() const {return this->_renderer;}

            template <class T>
            size_t push(Priority priority, T & object, Module<T> * modul) {
                if(this->_objects.contains(priority)) {
                    this->_objects.erase(priority);
                }
                this->_objects.insert({priority, Renderable{ object, modul }});
                return this->_objects.at(priority).id();
            }

            void remove(const Identifiable & identifiable) {
                for(auto & [priority, renderable] : this->_objects) {
                    if(renderable.id() == identifiable.id()) {
                        this->_objects.erase(priority);
                        break;
                    }
                } 
            }

            std::optional<std::reference_wrapper<const Renderable>> get(const Identifiable & id) const {
                for(const auto & [priority, object] : this->_objects) {
                    if(object.id() == id.id()) {
                        return object;
                    }
                }
                return {};
            }

            const Texture & draw(const Window & w, const TextureArray & a) {
                this->_frameBuffer.bind();
                this->_renderer.clear();
                for(auto & [type, object] : this->_objects) {
                    this->_renderer.render(w, object, a);
                }

                return this->_frameBuffer.texture();
            }

        private:
            FrameBuffer _frameBuffer;
            Renderer & _renderer;
            std::map<Priority, Renderable> _objects;            
    };
}