#include "Image_Module.hpp"

#include <engine/graphics/back/cameras/Camera.hpp>
#include <engine/utility/array_generator.hpp>

namespace megu {
    Image_Module::Image_Module()
    : _vbo(this->_vao, Image::Layout(), Image::Vertices(), EditMode::STATIC) {
        megu::Source vert_source("assets/shaders/Image-Instanced-Fat.vert", Source::Categorie::VERTEX);
        this->_program << vert_source;

        megu::Source frag_source("assets/shaders/Texture-Fat.frag", Source::Categorie::FRAGMENT);
        this->_program << frag_source;

        this->_program.link();
        vert_source.release();
        frag_source.release();
    }

    void Image_Module::draw(const ref_set<Image> & images, const Camera & camera, const Window & window) const {
        this->_vao.bind();
        this->_program.use();

        this->_program.setUniform("uProj", camera.projection());
        this->_program.setUniform("uView", camera.view());

        std::vector<std::reference_wrapper<const Texture>> textures;

        std::vector<glm::mat4> uModels;
        std::vector<GLint> uTextures;

        static auto uSampler = array_generator::Generate<GLint, 0, 32>();
        this->_program.setUniform("uSampler", uSampler);         
    
        for(auto & image : images) {
            std::vector<std::reference_wrapper<const Texture>>::iterator it = std::find(textures.begin(), textures.end(), image.get().texture());
            if(it != textures.end()) {
                uModels.push_back(image.get().transformation().model());
                uTextures.push_back(static_cast<GLint>(it - textures.begin()));
            }
            else {
                if(textures.size() >= 8 || uModels.size() >= 124) {
                    this->_program.setUniform("uModel", uModels);
                    this->_program.setUniform("uTextures", uTextures);

                    glDrawArraysInstanced(Quads::Primitive(), 0, static_cast<GLsizei>(this->_vbo.size()), static_cast<GLsizei>(uModels.size()));

                    textures.clear();
                    uModels.clear();
                    uTextures.clear();
                }
                
                textures.push_back(image.get().texture());  
                image.get().texture().bind(static_cast<GLint>(textures.size()-1));
                uTextures.push_back(static_cast<GLint>(textures.size()-1)); 
                uModels.push_back(image.get().transformation().model());
            }
        }

        if(!textures.empty()) {
            this->_program.setUniform("uModel", uModels);
            this->_program.setUniform("uTextures", uTextures);

            glDrawArraysInstanced(Quads::Primitive(), 0, static_cast<GLsizei>(this->_vbo.size()), static_cast<GLsizei>(uModels.size())); 
        }
    }
}