#include "TileArray_Module.hpp"

#include <engine/graphics/back/cameras/Camera.hpp>
#include <glm/gtc/matrix_transform.hpp>

namespace megu {
    TileArray_Module::TileArray_Module()
    : _vbo(this->_vao, TileArray::Layout(), TileArray::Vertices(), EditMode::STATIC) {
        Source vert_source("assets/shaders/TileArray.vert", Source::Categorie::VERTEX);
        this->_program << vert_source;

        Source frag_source("assets/shaders/TileArray.frag", Source::Categorie::FRAGMENT);
        this->_program << frag_source;

        this->_program.link();
        vert_source.release();
        frag_source.release();
    }

    void TileArray_Module::draw(const TileArray & vertexArray, const Camera & camera, const Window &) const {
        this->_vao.bind();
        this->_program.use();

        this->_program.setUniform("uProj", camera.projection());
        this->_program.setUniform("uView", camera.view());
        this->_program.setUniform("uModel", vertexArray.transformation().model());

        vertexArray.texture().bind();
        this->_program.setUniform("uSlot", 0);
        this->_program.setUniform("uSize", glm::vec2{vertexArray.texture().width(), vertexArray.texture().height()});

        std::vector<glm::mat4> uOffsets = {};
        std::vector<glm::vec4> uUvs = {};

        size_t count = 0;

        for(size_t x = 0; x < vertexArray.row(); ++x) {
            for(size_t y = 0; y < vertexArray.collumn(); ++y) {
                uOffsets.push_back(glm::translate(glm::mat4(1.f), {x, y, 0.f}));
                uUvs.push_back(vertexArray.uvs()[x][y]);

                if(count > 128) {
                    this->_program.setUniform("uOffsets", uOffsets);
                    this->_program.setUniform("uUvs", uUvs);
                    glDrawArraysInstanced(TileArray::Primitive(), 0, static_cast<GLsizei>(this->_vbo.size()), static_cast<GLsizei>(vertexArray.row() * vertexArray.collumn()));
                    
                    uOffsets.clear();
                    uUvs.clear();
                    count = 0;
                }
                ++count;
            }
        }

        if(count != 0) {
            this->_program.setUniform("uOffsets", uOffsets);
            this->_program.setUniform("uUvs", uUvs);
            //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
            glDrawArraysInstanced(TileArray::Primitive(), 0, static_cast<GLsizei>(this->_vbo.size()), static_cast<GLsizei>(vertexArray.row() * vertexArray.collumn()));
            //glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
        }
    }
}