#include "Image.hpp"

namespace megu {
    Image::Image(const std::filesystem::path & path) {
        this->load(path, true);
    }

    Image::Image(const Texture & texture) {
        this->link(texture);
    }

    Image Image::operator=(const Image & image) {
        this->_texture = image._texture;
        this->_transformation = image._transformation;
        
        return *this;
    }

    Image Image::operator=(const Image && image) {
        this->_texture = std::move(image._texture);
        this->_transformation = std::move(image._transformation);
        
        return *this;
    }

    void Image::load(const std::filesystem::path & path, bool flip) {
        TextureBuffer buffer;
        buffer.load(path, flip);
        this->load(buffer);
    }

    void Image::load(const TextureBuffer & buffer) {
        this->setSize(static_cast<float>(buffer.width()), static_cast<float>(buffer.height()));
        this->_texture.store(buffer);
    }

    void Image::link(const Texture & texture) {
        this->setSize(static_cast<float>(texture.width()), static_cast<float>(texture.height()));
        this->_texture = texture;
    }
}