#pragma once

#include <utility/Identifiable.hpp>

#include <filesystem>
#include <glm/vec2.hpp>

#include <engine/graphics/back/geometry/Transformable.hpp>
#include <engine/graphics/back/textures/Texture.hpp> 
#include <engine/graphics/front/geometry/Quads.hpp>
#include <engine/utility/type.hpp>

namespace megu {
    class Image : public Quads {
        public:
            Image() = default;
            Image(const std::filesystem::path &);
            Image(const Texture &);
            Image(const Image &) = default;
            Image operator=(const Image &);
            Image operator=(const Image &&);
   virtual ~Image() = default;

            inline Vec2  getPosition()  const {return Vec2(this->_transformation.x(), this->_transformation.y());}
            inline Vec2  getOrigine()   const {return Vec2(this->_transformation.origine().x, this->_transformation.origine().y);}
            inline Vec2  getSize()      const {return Vec2(this->_transformation.scaling().x, this->_transformation.scaling().y);}
            inline float getRotation()  const {return this->_transformation.roll();}

            inline const Texture        & texture()         const {return this->_texture;}
            inline const Transformable  & transformation()  const {return this->_transformation;}   
           
            inline void setPosition(float x, float y)   {this->_transformation.setPosition(x, y);}
            inline void setOrigine(float x, float y)    {this->_transformation.setOrigine(x, y);}
            inline void setSize(float w, float h)       {this->_transformation.setScaling(w, h);}
            inline void setRotation(float a)            {this->_transformation.setRotation(a, Transformable::Axis::Z);}

            inline void move(float x, float y)  {this->_transformation.move({x, y, 0.f});}
            inline void scale(float w, float h) {this->_transformation.scale({w, h, 0.f});}
            inline void rotate(float a)         {this->_transformation.rotate(a, Transformable::Axis::Z);}

            void load(const std::filesystem::path &, bool = false);
            void load(const TextureBuffer &);
            void link(const Texture &);

        private:
            Texture _texture;
            Transformable _transformation;
    };
}