#pragma once

#include <memory>
#include <iostream>

#include <engine/graphics/back/cameras/Camera.hpp>
#include <engine/graphics/front/module/Module.hpp>
#include <engine/graphics/front/engine/TextureArray.hpp>
#include <engine/io/Window.hpp>

#include <utility/Identifiable.hpp>

namespace megu {
    class Renderable : virtual public Identifiable {
        public:
           Renderable() = delete;

            template <class T>
            Renderable(T & object, Module<T> * module) 
            : _pimpl(std::make_unique<RenderableModel<T>>(object, module)) {}

            Renderable(const Renderable & src) {
                this->_pimpl = src._pimpl->clone();
            }

            Renderable & operator=(const Renderable & src) {
                this->_pimpl = src._pimpl->clone();
                return *this;
            }

            Renderable(Renderable &&) = default;
            Renderable & operator=(Renderable &&) = default;

        private:
            struct RenderableConcept {
                virtual ~RenderableConcept() {};

                virtual void render(const Window &, const Camera &) const = 0;
                virtual std::unique_ptr<RenderableConcept> clone() const = 0;
                virtual const std::type_info & type() const = 0;
            };

            template <class T>
            struct RenderableModel : public RenderableConcept {
                RenderableModel(T & v, Module<T> * m) 
                : _objects{ v }, _module{ m } {}

                void render(const Window & window, const Camera & camera) const override {
                    this->_module->draw(this->_objects, camera, window);
                }

                std::unique_ptr<RenderableConcept> clone() const override {
                    return std::make_unique<RenderableModel>(*this);
                }

                const std::type_info & type() const {
                    return typeid(T);
                }

                T & _objects;
                Module<T> * _module;
            };
        
            friend void render_ext(Renderable & renderable, const Window & window, const Camera & camera, const TextureArray & array) {
                renderable._pimpl->render(window, camera);
            }

            std::unique_ptr<RenderableConcept> _pimpl;
    };
}