#pragma once

#include <utility/Identifiable.hpp>

#include <engine/graphics/back/geometry/Transformable.hpp>
#include <engine/graphics/back/textures/Texture.hpp> 
#include <engine/graphics/front/geometry/Quads.hpp>

#include <engine/utility/type.hpp>

namespace megu {
    class Sprite : public Quads {
        public:
            using Frame =  glm::vec4;

            Sprite();
            Sprite(const std::filesystem::path &, bool = true);
            Sprite(const std::filesystem::path &, const Frame &, bool = true);
   virtual ~Sprite() = default;

            inline const Vec2 & getPosition()   const {return Vec2(this->_transformation.x(), this->_transformation.y());}
            inline const Vec2 & getOrigine()    const {return Vec2(this->_transformation.origine().x, this->_transformation.origine().y);}
            inline const Vec2 & getSize()       const {return Vec2(this->_transformation.scaling().x, this->_transformation.scaling().y);}
            inline const Frame & getFrame()     const {return this->_frame;} 
            inline float getrotation()          const {return this->_transformation.rotation().z;}

            inline const Texture        & texture()         const {return this->_texture;}
            inline const Transformable  & transformation()  const {return this->_transformation;} 

            inline void setPosition(float x, float y)   {this->_transformation.setPosition(x, y);}
            inline void setOrigine(float x, float y)    {this->_transformation.setOrigine(x, y);}
            inline void setSize(float w, float h)       {this->_transformation.setScaling(w, h);}
            inline void setRotation(float a)            {this->_transformation.setRotation(a, Transformable::Axis::Z);}
            inline void setFrame(const Frame & frame)   {this->_frame = frame;}

            inline void setFrame(float x, float y, float w, float h) {this->_frame = Frame(x, y, w ,h);}
            
            inline void move(float x, float y)  {this->_transformation.move({x, y, 0.f});}
            inline void scale(float w, float h) {this->_transformation.scale({w, h, 0.f});}
            inline void rotate(float a)         {this->_transformation.rotate(a, Transformable::Axis::Z);}

            void load(const std::filesystem::path &, bool = false);
            void load(const TextureBuffer &);
            void link(const Texture &);

        private:
            Frame _frame;
            Texture _texture;
            Transformable _transformation;
    };
}