#pragma once

#include <utility/Identifiable.hpp>

#include <array>

#include <engine/graphics/back/geometry/Transformable.hpp>
#include <engine/graphics/back/textures/Texture.hpp> 
#include <engine/graphics/front/geometry/Quads.hpp>

#include <engine/utility/type.hpp>

namespace megu {
    class TileArray : public Quads {
        public:
            TileArray() = delete;
            TileArray(const std::filesystem::path &, size_t, size_t, float, size_t);
   virtual ~TileArray() = default;

            inline Vec2 getPosition()   const {return Vec2(this->_transformation.x(), this->_transformation.y());}
            inline Vec2 getOrigine()    const {return Vec2(this->_transformation.origine().x, this->_transformation.origine().y);}
            inline Vec2 getSize()       const {return Vec2(this->_transformation.scaling().x, this->_transformation.scaling().y);}
            inline float getrotation()  const {return this->_transformation.rotation().z;}

            inline void setPosition(float x, float y)   {this->_transformation.setPosition(x, y);}
            inline void setOrigine(float x, float y)    {this->_transformation.setOrigine(x, y);}
            inline void setSize(float w, float h)       {this->_transformation.setScaling(w, h);}
            inline void setRotation(float a)            {this->_transformation.setRotation(a, Transformable::Axis::Z);}
            inline void setLayer(float l)               {this->_transformation.setZ(l);}

            inline void move(float x, float y)  {this->_transformation.move(x, y);}
            inline void scale(float w, float h) {this->_transformation.scale(w, h);}
            inline void rotate(float a)         {this->_transformation.rotate(a, Transformable::Axis::Z);}

            inline size_t row() const {return this->_row;}
            inline size_t collumn() const {return this->_col;}
            inline float tileSize() const {return this->_tileSize;}

            inline void setUv(size_t x, size_t y, const glm::vec4 & uv) {this->_uvs[y][x] = uv;}
            inline const glm::vec4 & getUv(size_t x, size_t y) const {return this->_uvs[y][x];}

            inline const Transformable & transformation()  const {return this->_transformation;}
            inline const Texture & texture() const {return this->_texture;}

            inline GLint getTextureWidth() const {return this->_texture.width();}
            inline GLint getTextureHeight() const {return this->_texture.height();}

            inline const std::vector<std::vector<glm::vec4>> & uvs() const {return this->_uvs;}
            std::vector<glm::vec4> & operator[](size_t index) {return this->_uvs[index];} 

        private:
            Transformable _transformation;
            Texture _texture;
            size_t _row, _col;
            float _tileSize;
            std::vector<std::vector<glm::vec4>> _uvs;
    };
}