#include "Profiler.hpp"

#include <functional>

namespace megu {
    void Profiler::set(Keyboard * keyboard) {
        this->_keyboard = std::unique_ptr<Keyboard>(keyboard);
    }

    void Profiler::set(Mouse * mouse) {
        this->_mouse = std::unique_ptr<Mouse>(mouse);
    }

    void Profiler::link(const Window & window) const {
        glfwSetWindowUserPointer(window.ptr(), (void *)(this));
        if(this->_keyboard != nullptr) {
            glfwSetKeyCallback(window.ptr(), [](GLFWwindow * window, int key, int scancode, int action, int mods) {
                static_cast<Profiler *>(glfwGetWindowUserPointer(window))->keyboard()->on(Keyboard::Key(key), scancode, Keyboard::Action(action),Keyboard::Mod(mods));
            });
        }

        if(this->_mouse != nullptr) {
            glfwSetCursorPosCallback(window.ptr(), [](GLFWwindow * window, double x, double y) {
                static_cast<Profiler *>(glfwGetWindowUserPointer(window))->mouse()->on(x, y);
            });

            glfwSetCursorEnterCallback(window.ptr(), [](GLFWwindow * window, int entered) {
                static_cast<Profiler *>(glfwGetWindowUserPointer(window))->mouse()->on(entered);
            });

            glfwSetMouseButtonCallback(window.ptr(), [](GLFWwindow * window, int button, int action, int mods) {
                static_cast<Profiler *>(glfwGetWindowUserPointer(window))->mouse()->on(Mouse::Button(button), Mouse::Action(action), Mouse::Mod(mods));
            });
        }
    }
}