#include "Window.hpp"

#include <stdexcept>

namespace megu {
    Window::Window() 
    : _pointer(nullptr) {
        if(glfwInit() == GLFW_FALSE) {
            throw std::runtime_error("Cannot initialize GLFW.");
        }
    }

    uint16_t Window::width() const {
        if(this->_pointer != nullptr) {
            int width;
            glfwGetWindowSize(this->_pointer, &width, NULL);
            return width;
        }
        return 0;
    }

    uint16_t Window::height() const {
         if(this->_pointer != nullptr) {
            int height;
            glfwGetWindowSize(this->_pointer, NULL, &height);
            return height;
        }
        return 0;
    }

    bool Window::isOpen() const {
        return this->_pointer != nullptr ? !glfwWindowShouldClose(this->_pointer) : false;
    }

    void Window::resize(uint16_t width, uint16_t height) {
        if(this->_pointer != nullptr) {
            glfwSetWindowSize(this->_pointer, width, height);
        }
    }

    void Window::setTitle(const std::string & title) {
        if(this->_pointer != nullptr) {
            glfwSetWindowTitle(this->_pointer, title.c_str());
        }
    }

    void Window::open(const std::string & title, uint16_t width, uint16_t height) {
        if(this->_pointer == nullptr) {
            this->_pointer = glfwCreateWindow(width, height, title.c_str(), NULL, NULL);
            if(this->_pointer== NULL) {
                throw std::runtime_error("Cannot open Window.");
            }

            glfwMakeContextCurrent(this->_pointer);
            glfwSwapInterval(0);

            if(glewInit()) {
                throw std::runtime_error("Cannot initialize GLEW.");
            }

            glViewport(0, 0, width, height);
        }
    }

    void Window::close() {
        if(this->_pointer != nullptr) {
            glfwSetWindowShouldClose(this->_pointer, true);
        }
    }

    void Window::pollEvents() {
        glfwPollEvents();
    }

    void Window::swapBuffers() {
        if(this->_pointer != nullptr) {
            glfwSwapBuffers(this->_pointer);
        }
    }
}