#include "Position.hpp"

namespace megu {
    Position::Position() 
    : Position(0.f, 0.f) {}

    Position::Position(float x, float y, float z)
    : _position(x, y, z) {}

    Position::Position(const glm::vec3 & vec) 
    : _position(vec) {}

    Position::Position(const Position & src)
    : _position(src._position) {}

    Position Position::operator=(const Position & src) {
        this->_position = src._position;
        return *this;
    }

    void Position::move(const glm::vec3 & direction) {
        this->_position.x += direction.x;
        this->_position.y += direction.y;
        this->_position.z += direction.z;
    }

    bool Position::operator==(const Position & position) const {
        return this->_position == position._position;
    }

    bool Position::operator!=(const Position & position) const {
        return !(*this == position);
    }

    Position operator+(const Position & pa, const Position & pb) {
        return Position(pa.x() + pb.x(), pa.y() + pb.y(), pa.z() + pb.z());
    }

    Position operator-(const Position & pa, const Position & pb) {
        return Position(pa.x() - pb.x(), pa.y() - pb.y(), pa.z() - pb.z());
    }

    Position operator*(const Position & pa, const Position & pb) {
        return Position(pa.x() * pb.x(), pa.y() * pb.y(), pa.z() * pb.z());
    }

    Position operator/(const Position & pa, const Position & pb) {
        return Position(pa.x() / pb.x(), pa.y() / pb.y(), pa.z() / pb.z());
    }

    bool operator<(const Position & pa, const Position & pb) {
        return pa.x() < pb.x() && pa.y() < pb.y() && pa.z() < pb.z();
    }

    bool operator>(const Position & pa, const Position & pb) {
        return pa.x() > pb.x() && pa.y() > pb.y() && pa.z() > pb.z();
    }

    bool operator<=(const Position & pa, const Position & pb) {
        return pa.x() <= pb.x() && pa.y() <= pb.y() && pa.z() <= pb.z();
    }

    bool operator>=(const Position & pa, const Position & pb) {
        return pa.x() >= pb.x() && pa.y() >= pb.y() && pa.z() >= pb.z();
    }
}