#include "SquareBox.hpp"

#include <iostream>

namespace megu {
    SquareBox::SquareBox(const Position & position, const Dimension & dimension) 
    : _position(position), _dimension(dimension) {}

    SquareBox::SquareBox(const SquareBox & src)
    : _position(src._position), _dimension(src._dimension) {}

    SquareBox SquareBox::operator=(const SquareBox & src) {
        this->_position = src._position;
        this->_dimension = src._dimension;
        return *this;
    }

    void SquareBox::move(const Direction & direction) {
        this->_position.move(direction.x(), direction.y(), direction.z());
    }

    bool SquareBox::intersect(const SquareBox & squareBox) const {
        const Position & a = squareBox._position;
        const Dimension & as = squareBox._dimension;
        const Position & b = this->_position;
        const Dimension & bs = this->_dimension;

        bool x = std::max(a.x(), b.x()) < std::min(a.x() + as.x, b.x() + bs.x);
        bool y = std::max(a.y(), b.y()) < std::min(a.y() + as.y, b.y() + bs.y);

        bool z = true;
        if(a.z() != 0 && b.z() != 0) {
            z = std::max(a.z(), b.z()) < std::min(a.z() + as.z, b.z() + bs.z);
        }

        return x && y && z;
    }

    bool SquareBox::operator==(const SquareBox & squareBox) const {
        return this->_position == squareBox._position && this->_dimension == squareBox._dimension;
    }

    bool SquareBox::operator!=(const SquareBox & squareBox) const {
        return !(*this == squareBox);
    }

    bool SquareBox::operator|(const SquareBox & squareBox) const {
        return this->intersect(squareBox);
    }
}