#pragma once

#include <array>

#include "Position.hpp"

namespace megu {
    using Dimension = glm::vec3; 

    class SquareBox {
        public:
            SquareBox() = delete;
            SquareBox(const Position &, const Dimension &);
            SquareBox(const SquareBox &);
            SquareBox operator=(const SquareBox &);

            inline const Position & getPosition() const {return this->_position;}
            inline const Dimension & getDimension() const {return this->_dimension;}

            inline float x() const {return this->_position.x();}
            inline float y() const {return this->_position.y();}
            inline float z() const {return this->_position.z();}

            inline float width()  const {return this->_dimension.x;}
            inline float height() const {return this->_dimension.y;}
            inline float depth()  const {return this->_dimension.z;}

            inline void setPosition(const Position & position) {this->_position = position;}
            inline void setPosition(float x, float y, float z = 0.f) {this->_position = Position(x, y, z);}

            inline void setDimension(const Dimension & dimension) {this->_dimension = dimension;}
            inline void setDimension(float x, float y, float z = 0.f) {this->_dimension = Dimension(x, y, z);}

            void move(const Direction &);
            inline void move(float x, float y, float z = 0.f) {this->move(Direction(x, y, z));}

            bool intersect(const SquareBox &) const;

            bool operator==(const SquareBox &) const;
            bool operator!=(const SquareBox &) const;
            bool operator|(const SquareBox &)  const;

        private:
            Position _position;
            Dimension _dimension;
    }; 
}