#pragma once

#include <vector>
#include <map>
#include <set>
#include <optional>

#include <engine/physic/front/object/Tangible.hpp>
#include <engine/physic/front/object/TangibleStatic.hpp>
#include <engine/physic/front/object/TangibleMovable.hpp>
#include <engine/physic/front/object/Collision.hpp>

#include <engine/utility/Priority.hpp>
#include <engine/utility/reference_sorter.hpp>
#include <engine/utility/ref_set.hpp>

namespace megu {
    class PhysicEngine {
        public:
            PhysicEngine();

            void push(Priority, TangibleStatic &);
            void push(Priority, TangibleMovable &);

            void remove(TangibleStatic &);
            void remove(TangibleMovable &);

            void step(double);
            void step(double, Priority);

            std::optional<SquareBox> checkCollision(const SquareBox &, Priority) const;

            std::optional<std::reference_wrapper<const Tangible>> get(const Identifiable &) const;

            inline void clearCollision() {this->_collisions.clear();}

            const std::set<Collision, reference_sorter<Collision>> & collision() const {return this->_collisions;}

        private:
            std::map<Priority, std::vector<std::reference_wrapper<TangibleStatic>>> _statics;
            std::map<Priority, std::vector<std::reference_wrapper<TangibleMovable>>> _dynamic;
            std::set<Collision, reference_sorter<Collision>> _collisions; 
    };
}