#pragma once

#include <utility/Identifiable.hpp>

#include <engine/physic/back/Position.hpp>
#include <engine/physic/back/SquareBox.hpp>
#include <functional>
#include <optional>

namespace megu {
    class Tangible : virtual public Identifiable {
        public:
            Tangible();
            Tangible(const Position &, const Dimension &);
   virtual ~Tangible() = default;

            inline const Position & getPosition() const {return this->_box.getPosition();}
            inline const SquareBox & getBox() const {return this->_box;}

            inline float x() const {return this->_box.getPosition().x();}
            inline float y() const {return this->_box.getPosition().y();}
            inline float z() const {return this->_box.getPosition().z();}

            inline void setPosition(float x, float y, float z = 0.f) {this->_box.setPosition(x, y, z);}
            inline void setDimension(float x, float y, float z = 0.f) {this->_box.setDimension(x, y, z);}

            inline void move(const Direction & direction) {this->_box.move(direction);}
            inline void move(float x, float y, float z = 0.f) {return this->move(Direction(x, y, z));}

            virtual std::optional<SquareBox> isColliding(const Tangible &) const;
            virtual std::optional<SquareBox> isColliding(const SquareBox &) const;

            bool operator==(const Tangible &) const;
            std::optional<SquareBox> operator|(const Tangible &) const;
            std::optional<SquareBox> operator|(const SquareBox &) const;

            using UpdateLambda = std::function<void(double)>;

        protected:
            virtual void update(double) = 0;

        private:
            SquareBox _box;
    };

}