#include "Game.hpp"


#include <kernel/front/Kernel.hpp>

#include <game/back/object/Player.hpp>
#include <game/back/object/Enemy.hpp>
#include <game/back/object/Level.hpp>
#include <game/back/object/Terrain.hpp>
#include <game/back/object/tile/TileSolide.hpp>

#include <game/front/Layer.hpp>
#include <game/front/object/Klinck.hpp>
#include <game/utility/FrameCouter.hpp>

namespace megu::game {
    Game::Game(const std::string & title)
    : _title(title) {
       
    }

    bool Game::run() {
        try {
            Window window;
            window.open(this->_title, 720, 720);
            FrameCounter counter;

            kernel::Kernel kernel(window);
                        
            Klinck klinck(kernel, 170, 170);

            Terrain terrain(0.f, 0.f, 32.f, 32.f, "assets/game/levels/BSZ/tilemap.png", 11, 11, 32.f, 16);

            std::vector<size_t> layer0 = {
                48, 49, 49, 23, 30, 31, 32, 34, 50, 50, 51,
                54, 55, 55, 29, 36, 37, 38, 40, 56, 56, 57,
                47,  1,  1,  1,  1,  1,  1,  1,  1,  1, 46,
                47,  1,  1,  1,  1,  1,  1,  1,  1,  1, 46,
                47,  1,  1,  1,  1,  1,  1,  1,  1,  1, 46,
                47,  1,  1,  1,  1,  1,  1,  1,  1,  1, 46,
                47,  1, 12, 13,  1,  1,  1, 12, 13,  1, 46,
                23,  1,  1,  1,  1,  1,  1,  1,  2,  1, 34,
                29,  1,  1,  1,  1,  1,  1,  1,  1,  1, 40,
                18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
                24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
            };

            for(size_t i = 7; i < 60; ++i) {
                terrain.setTileEvent(i, new TileSolide(32.f, physic::TERRAIN));
            }
            terrain.setTileEvent(2, new TileSolide(32.f, physic::TERRAIN));

            for(size_t i = 0; i < 11; ++i) {
                terrain.graphic().pushAnimation(i, 9, {18, 19, 20, 21});
                terrain.graphic().pushAnimation(i, 10, {24, 25, 26, 27});
            }

            for(size_t i = 0; i < 11; ++i) {
                for(size_t j = 0; j < 11; ++j) {
                    terrain.setValue(j, i, layer0.at(i*11 + j));
                }
            }
            
            terrain.getGraphicComponent()->setLayerPriority(graphic::TERRAIN);
            terrain.getPhysicComponent()->setLayer(physic::TERRAIN);

            Terrain terrain_2(0.f, 0.f, 32.f, 32.f, "assets/game/levels/BSZ/tilemap.png", 11, 11, 32.f, 16);

            std::vector<size_t> layer1 = {
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  0,  0,  9,  0,  0,  0, 14,  0,  0,  0,
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  0,  6,  7,  0,  0,  0,  6,  7,  0,  0,
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  15,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
            };

            terrain_2.setTileEvent(9, new TileSolide(32.f, physic::TERRAIN));
            terrain_2.setTileEvent(14, new TileSolide(32.f, physic::TERRAIN));
            terrain_2.setTileEvent(15, new TileSolide(32.f, physic::TERRAIN));

            for(size_t i = 0; i < 11; ++i) {
                for(size_t j = 0; j < 11; ++j) {
                    terrain_2.setValue(j, i, layer1.at(i*11 + j));
                }
            }

            terrain_2.getGraphicComponent()->setLayerPriority(graphic::TERRAIN_UP);
            terrain_2.getPhysicComponent()->setLayer(physic::TERRAIN);
            

            Level level("BSZ");

            terrain.setup(kernel, level);
            terrain.apply(kernel);

            terrain_2.setup(kernel, level);
            terrain_2.apply(kernel);
            
            //level.add(&enemy);

            level.add(&klinck);
            klinck.setup(kernel, level);
            klinck.apply(kernel);

            //enemy.setup(kernel, level);
            //enemy.apply(kernel);

            std::cout << "..." << std::endl;

            while(window.isOpen()) {
                counter.count(Window::Time());
                window.pollEvents();
                kernel.step();
            }
        }
        catch(std::exception & error) {
            std::cerr << "[Error] : " << error.what() << std::endl;
            return EXIT_FAILURE;
        }

        return EXIT_SUCCESS;
    }
}