#include "Enemy.hpp"

#include <kernel/front/Kernel.hpp>
#include <game/back/object/Level.hpp>

#include <game/front/Stats.hpp>

namespace megu::game {
    Enemy::Enemy(float x, float y, float w, float h, std::filesystem::path & path) 
    : kernel::PropsDynamic(this->_sprite, this->_movable), GameProps(this), _sprite(path), _movable(x, y, w, h) {
        this->_sprite.setPosition(x, y);
    }

    void Enemy::setup(kernel::Kernel & kernel, Level & level) {
        this->_sprite.setFrame(0.f, 0.f, 51.f, 98.f);
        this->_sprite.setSize(51.f, 98.f);

        this->_movable.setCollideLambda([this, &level](kernel::Kernel &, kernel::PhysicEngine &, Identifiable & id, kernel::Physical<kernel::PhysicEngine> & comp, double) {
            auto object = level.get(id);
            if(object.has_value()) {
                auto event = object->get().on();
                if(event.has_value()) {
                    this->on(*object->get().parent(), event.value());
                }  
            }
        });
    }

    void Enemy::destroy(kernel::Kernel & kernel, Level & level) {
        
    }

    void Enemy::on(const kernel::Prop &, const Event & event) {
        if(event & Event::Type::SOLID) {
            this->onSolide(event);
        }

        if(event & Event::Type::DAMAGE) {
            this->onDamage(event);
        }
    }

    std::optional<Event> Enemy::on() const {
        Event b(Event::DAMAGE);
        b.set(Stats::ATK, 10);

        return b;
    }

    void Enemy::apply(kernel::Kernel & kernel) {
        kernel.push(*this);
    }

    void Enemy::onDamage(const Event &) {
        std::cout << "Enemy Got Damage !" << std::endl;
    }

    void Enemy::onSolide(const Event &) {
        std::cout << "Enemy Got Solide !" << std::endl;
    }
}