#include "Player.hpp"
#include <game/back/object/Level.hpp>

namespace megu::game {
    Player::Player(float x, float y, float w, float h, std::filesystem::path path)
    : kernel::PropsPlayable(this->_sprite, this->_movable), GameProps(this), _sprite(path), _movable(x, y, w, h) {}

    void Player::move(float x, float y) {
        this->_sprite.move(x, y);
        this->_movable.move(x, y);
    }

    void Player::setPosition(float x, float y) {
        this->_sprite.setPosition(x, y);
        this->_movable.setPosition(x, y);
    }

    const Position & Player::getPosition() const {
        return this->_movable.getPosition();
    }

    void Player::setup(kernel::Kernel & kernel, Level & level) {
        this->_movable.setCollideLambda([this, &level](kernel::Kernel &, kernel::PhysicEngine &, Identifiable & id, kernel::Physical<kernel::PhysicEngine> & comp, double) {
            auto object = level.get(id);

            if(object.has_value()) {
                auto event = object.value().get().on();  
                if(event.has_value()) {
                    this->on(*object.value().get().parent(), event.value());
                }
            }
        });

        this->_movable.setUpdateLambda([this](double delta) {
            this->onPhysic(delta);
        });

        this->onSetup(kernel, level);
    }

    void Player::on(const kernel::Prop & props, const Event & event) {
        if((event & Event::Type::SOLID) != 0) {
            this->onSolide(props);
        }

        if((event & Event::Type::DAMAGE) != 0) {
            this->onDamage(event);
        }
    }
    
    void Player::destroy(kernel::Kernel & kernel, Level & level) {
        this->onDestroy(kernel, level);
    }

    void Player::apply(kernel::Kernel & kernel) {
        kernel.push(*this);
    }
}