#pragma once

#include <kernel/front/props/PropsPlayable.hpp>
#include <game/back/GameObject.hpp>
#include <game/back/object/Level.hpp>

namespace megu::game {
    class Player : public kernel::PropsPlayable, public GameProps {
        public:
            Player(float x, float y, float w, float h, std::filesystem::path);

            void setup(kernel::Kernel &, Level &) override final;
            void destroy(kernel::Kernel &, Level &) override final;
            
            void setPosition(float, float);
            void move(float, float);

            const Position & getPosition() const;
            
            void on(const kernel::Prop &, const Event &) override final;
            void apply(kernel::Kernel &) override final;
            
            std::optional<Event> on() const = 0;

            virtual void onDamage(const Event &) = 0;
            virtual void onSolide(const kernel::Prop &) = 0; 
            virtual void onSetup(kernel::Kernel &, Level &) = 0;
            virtual void onDestroy(kernel::Kernel &, Level &) = 0;
            virtual void onPhysic(double) = 0;

            kernel::Movable & getPhysic() {return this->_movable;}
            kernel::Sprite & getGraphic() {return this->_sprite;}

        private:
            kernel::Sprite _sprite;
            kernel::Movable _movable;
    };
}