#include "Terrain.hpp"

#include <game/back/object/Level.hpp>


namespace megu::game {
    Terrain::Terrain(float x, float y, float w, float h, const std::filesystem::path & path, size_t r, size_t c , float size, size_t tileSize)
    : PropsTileMap(this->_graphic, this->_physic), GameProps(this), _graphic(path, r, c, size, tileSize), _physic(x, y, r * w, c * h) {
        //! Taille Pour 1 tile !
        this->_graphic.setSize(w, h);
    }

    void Terrain::setValue(size_t x, size_t y, size_t value) {
        this->_graphic.setValue(x, y, value);
        if(this->_event.contains(value)) {
            this->_physic.push(
                {
                    static_cast<float>(x) * this->_graphic.getSize().x, 
                   (this->_graphic.getSize().y * (this->_graphic.collumn()- 1)) - static_cast<float>(y) * this->_graphic.getSize().y
                }, 
                *this->_event[value]
            );
        }
    }

    void Terrain::setTileEvent(size_t id, Tile * tile) {
        this->_event[id] = std::shared_ptr<Tile>(tile);
    }

    void Terrain::setup(kernel::Kernel & kernel, Level & level) {
        this->_physic.setCollideLambda([this, &level](kernel::Kernel &, kernel::PhysicEngine &, Identifiable & id, kernel::Tile & tile, double) {
            auto it = this->_event.begin();
            for(; it != this->_event.end(); ++it) {
                if(*it->second == tile) {
                    break;
                }
            }

            if(it != this->_event.end()) {
                auto object = level.get(id);
                auto event = it->second->on();
                
                if(event.has_value() && object.has_value()) {
                    object.value().get().on(*this, event.value());
                }
            }
        });
    }

    void Terrain::destroy(kernel::Kernel & kernel, Level & level) {
        this->_event.clear();
    }

    void Terrain::apply(kernel::Kernel & kernel) {
        kernel.push(*this);
    }

    void Terrain::on(const kernel::Prop &, const Event & event) {
        // The terrain itself don't react.
    }

    std::optional<Event> Terrain::on() const {
        return {};
    }
}