#include "Klinck.hpp"

#include <game/front/Stats.hpp>
#include <game/front/Layer.hpp>
#include <game/front/profile/KlinckKeys.hpp>

namespace megu::game {
    Klinck::Klinck(kernel::Kernel & k, float x, float y)
    : Player(0.f, 0.f, 32.f, 32.f, "assets/game/objects/klinck.png"), Damagable(100), _kernel(k), _atk(10), _def(25), _vx(0.f), _vy(0.f) {
        this->getGraphic().setLayerObject(graphic::Layer::ENTITY);
        this->getPhysic().setLayer(physic::Layer::ENTITY);
        this->move(x, y);
    }

    void Klinck::onSetup(kernel::Kernel & kernel, Level &) {
        auto & sprite = this->getGraphic();

        sprite.pushAnimation(Animation::IDL_BACK,     0,  32, 16, 16);
        sprite.pushAnimation(Animation::IDL_FRONT,    16, 32, 16, 16);
        sprite.pushAnimation(Animation::IDL_LEFT,     32, 32, 16, 16);
        sprite.pushAnimation(Animation::IDL_RIGHT,    48, 32, 16, 16);

        sprite.pushAnimation(Animation::WALK_BACK,    {{0,  16, 16, 16}, { 16, 16, 16, 16}});
        sprite.pushAnimation(Animation::WALK_FRONT,   {{32,  16, 16, 16}, { 48, 16, 16, 16}});
        sprite.pushAnimation(Animation::WALK_LEFT,    {{0, 0, 16, 16}, { 16, 0, 16, 16}});
        sprite.pushAnimation(Animation::WALK_RIGHT,   {{32, 0, 16, 16}, { 48, 0, 16, 16}});

        sprite.setCurrentAnimation(Animation::IDL_BACK);
        sprite.setSize(32.f, 32.f);

        this->setControl(kernel.window(), new KlinckKeyProfile(*this, kernel));

        this->_moving[UP] = false;
        this->_moving[DOWN] = false;
        this->_moving[LEFT] = false;
        this->_moving[RIGHT] = false;
    }

    void Klinck::onDestroy(kernel::Kernel &, Level &) {

    }

    void Klinck::onDamage(const Event & event) {
        this->damage(event.get(Stats::ATK).value_or(0));
    }

    void Klinck::onSolide(const kernel::Prop &) {

    }

    void Klinck::onPhysic(double) {
        if(abs(this->_vx) > 0.05f) {
            this->_vx = this->_vx > 0 ? 0.05f : -0.05f;
        }

        if(abs(this->_vy) > 0.05f) {
            this->_vy = this->_vy > 0 ? 0.05f : -0.05f;
        }

        float vx = 0.f, vy = 0.f;
        if(this->_moving[UP]) {
            vy += 0.01f;
        }

        if(this->_moving[DOWN]) {
            vy -= 0.01f;
        }

        if(this->_moving[RIGHT]) {
            vx += 0.01f;
        }

        if(this->_moving[LEFT]) {
            vx -= 0.01f;
        }

        this->_vx += vx;
        this->_vy += vy;

        if(abs(this->_vy) > 0) {
            SquareBox box = this->getPhysic().getBox();
            box.move({0.f, this->_vy, 0.f});

            std::optional<SquareBox> obox = this->_kernel.getPhysicEngine().get().checkCollision(box, physic::TERRAIN);
            if(obox.has_value()) {
                float pos_y = this->_vy > 0 ? obox.value().y() - obox.value().height() : obox.value().y() + obox.value().height();
                this->setPosition(this->getPosition().x(), pos_y);
                this->_vy = 0.f;
            }
            else {
                this->move(0.0, this->_vy);
            }
        }

        if(abs(this->_vx) > 0) {
            SquareBox box = this->getPhysic().getBox();
            box.move({this->_vx, 0.f, 0.f});

            std::optional<SquareBox> obox = this->_kernel.getPhysicEngine().get().checkCollision(box, physic::TERRAIN);
            if(obox.has_value()) {
                float pos_x = this->_vx > 0 ? obox.value().x() - obox.value().width() : obox.value().x() + obox.value().width();
                this->setPosition(pos_x, this->getPosition().y());
                this->_vx = 0.f;
            }
            else {
                this->move(this->_vx, 0.0);
            }
        }

        if(abs(this->_vy) > 0.001f) {
            this->_vy += this->_vy > 0.f ? -0.0002f : 0.0002f;
        }
        else {
            this->_vy = 0.f;
        } 

        if(abs(this->_vx) > 0.001f) {
            this->_vx += this->_vx > 0.f ? -0.0002f : 0.0002f;
        }
        else {
            this->_vx = 0.f;
        } 
    }

    std::optional<Event> Klinck::on() const {
        Event event(Event::DAMAGE);
        event.set(Stats::ATK, this->_atk);

        return event;
    }
}