#pragma once

#include <game/back/object/Player.hpp>
#include <game/back/object/Damagable.hpp>

namespace megu::game {
    class Klinck : public Player, public Damagable {
        public:
            enum Animation {
                IDL_FRONT = 1,
                IDL_BACK  = 2,
                IDL_LEFT  = 3,
                IDL_RIGHT = 4,

                WALK_FRONT = 5,
                WALK_BACK  = 6,
                WALK_LEFT  = 7,
                WALK_RIGHT = 8
            };

            enum Direction {
                UP,
                DOWN,
                LEFT,
                RIGHT
            };

            Klinck(kernel::Kernel &, float, float);

            void onSetup(kernel::Kernel &, Level &) override;
            void onDestroy(kernel::Kernel &, Level &) override;
            void onDamage(const Event &) override;
            void onSolide(const kernel::Prop &) override;
            void onPhysic(double) override;
            std::optional<Event> on() const override;

            inline void setVelocity(float x, float y) {this->_vx = x; this->_vy = y;}
            inline void addVelocity(float x, float y) {this->_vx += x; this->_vy += y;}
            inline float vx() const {return this->_vx;}
            inline float vy() const {return this->_vy;}

            inline void setMovingDirection(Direction dir, bool state) {this->_moving[dir] = state;}

        private:
            kernel::Kernel & _kernel;
            unsigned int _atk;
            unsigned int _def;
            float _vx, _vy;
            std::map<Direction, bool> _moving;
    };
}