#pragma once

#include "Engine.hpp"

#include <engine/io/Window.hpp>

#include <engine/graphics/front/engine/Engine.hpp>
#include <engine/graphics/front/engine/Renderer.hpp>

#include <engine/graphics/front/module/TileArray_Module.hpp>
#include <engine/graphics/front/object/TileArray.hpp>

#include <kernel/back/component/Graphical.hpp>

namespace megu::kernel {
    class GraphicEngine : public Engine<megu::GraphicEngine, Graphical<GraphicEngine>, Renderable> {
        public:
            GraphicEngine(Window &);

            inline megu::GraphicEngine & get() {return this->_engine;}

            void boot() override;
            void stop() override;
            void step(double) override;

            void push(Graphical<GraphicEngine> &) override;
            void remove(Graphical<GraphicEngine> &) override;

            const megu::Renderable & get(const Graphical<GraphicEngine> &) const override;

            inline const megu::GraphicEngine & engine() const {return this->_engine;}
            inline const megu::Renderer & renderer() const {return this->_renderer;}

        private:
            megu::GraphicEngine _engine;
            megu::Renderer _renderer;
    };
}