#include "GraphicLinker.hpp"

namespace megu::kernel {
    template <class T>
    GraphicLinker<T>::GraphicLinker()
    : _module(nullptr) {}

    template <class T>
    void GraphicLinker<T>::link(T & object, Priority l, Priority o) {
        auto layers = RenderTrace{l, o};
        this->_stored[layers].insert(object);
        this->_await.insert(layers);
    }

    template <class T>
    void GraphicLinker<T>::link(GraphicEngine & engine) {
        for(const auto & layer : this->_delete) {
            engine.get().remove(Identifiable(this->_ids[layer]));
        }

        this->_delete.clear();

        for(auto & layer : this->_await) {
            size_t id = engine.get().push(layer.layer, layer.object, this->_stored[layer], this->_module.get());
            this->_ids[layer] = id;
        }

        this->_await.clear();
    }

    template <class T>
    void GraphicLinker<T>::unlink(T & object) {
        for(auto it = this->_stored.begin(); it != this->_stored.end();) {
            bool contains = false;
            for(auto its = it->second.begin(); its != it->second.end();) {
                if(&(its->get()) == &object) {
                    contains = true;
                    it->second.erase(its);
                    break;
                }
                else {
                    ++its;
                }
            }

            if(contains) {
                this->_delete.insert(it->first);
                this->_await.insert(it->first);
                if(it->second.empty()) {
                    it = this->_stored.erase(it);
                }
                else {
                    ++it;
                }
                break;
            }
            else {
                ++it;
            }
        }
    }

    template <class T>
    void GraphicLinker<T>::setModule(Module<ref_set<T>> * mod) {
        this->_module = std::unique_ptr<Module<ref_set<T>>>(mod);
    }
}
