#include "UniqueLinker.hpp"

namespace megu::kernel {
    template <class T>
    UniqueGraphicLinker<T>::UniqueGraphicLinker() 
    : _module(nullptr) {}

    template <class T>
    void UniqueGraphicLinker<T>::link(T & object, Priority l, Priority o) {
        auto layers = RenderTrace{l, o};
        this->_await.insert({layers, object});
    }

    template <class T>
    void UniqueGraphicLinker<T>::link(GraphicEngine & engine) {
        for(const auto & layer : this->_delete) {
            engine.get().remove(Identifiable(this->_ids[layer]));
        }

        this->_delete.clear();

        for(auto & [layer, object] : this->_await) {
            this->_stored.insert({layer, {object, 0}});
            size_t id = engine.get().push<T>(layer.layer, layer.object, this->_stored.at(layer).first.get(), this->_module.get());
            this->_ids[layer] = id;
        }

        this->_await.clear();
    }

    template <class T>
    void UniqueGraphicLinker<T>::unlink(T & object) {
        for(auto & [layer, obj] : this->_stored) {
            if(&(obj.first.get()) == &object) {
                this->_delete.insert(layer);
            }
        }
    }

    template <class T>
    void UniqueGraphicLinker<T>::setModule(Module<T> * mod) {
        this->_module = std::unique_ptr<Module<T>>(mod);
    }
}
