#include "Kernel.hpp"

namespace megu::kernel {
    Kernel::Kernel(Window & window) 
    : _window(window), _pEngine(), _gEngine(window) {
        this->_pEngine.boot();
        this->_gEngine.boot();
    }

    Kernel::~Kernel() {
        this->_pEngine.stop();
        this->_pEngine.stop();
    }

    void Kernel::step() {
        double time = Window::Time();

        if(this->_window.isOpen()) {
            this->_gEngine.step(time);
            this->_gResolver.resolve(*this, this->_gEngine, time);
        }

        this->_pEngine.step(time);
        this->_pResolver.resolve(*this, this->_pEngine, time);
    }

    void Kernel::push(Prop & props) {
        this->_props[props.id()] = &props;
        auto * pComponent = props.getPhysicComponent();

        if(pComponent != nullptr) {
            this->_pEngine.push(*pComponent);
            this->_pResolver.push(*pComponent);
        }
        
        auto * gComponent = props.getGraphicComponent();
        if(gComponent != nullptr) {
            this->_gEngine.push(*gComponent);
            this->_gResolver.push(*gComponent);
        }
    }

    void Kernel::remove(Prop & props) {
        auto * pComponent = props.getPhysicComponent();
        if(pComponent != nullptr) {
           this->_pEngine.remove(*pComponent);
           this->_pResolver.remove(*pComponent);
        }
        
        auto * gComponent = props.getGraphicComponent();
        if(gComponent != nullptr) {
            this->_gEngine.remove(*gComponent);
            this->_gResolver.remove(*gComponent);
        }

        this->_props.erase(props.id());
    }

    Prop * Kernel::get(const Prop::Physical_Component & pc) {
        for(auto & [id, props] : this->_props) {
            if(props->getPhysicComponent()->id() == pc.id()) {
                return props;
            }
        }
        return nullptr;
    }


    Prop * Kernel::get(const Prop::Graphical_Component & gc) {
        for(auto & [id, props] : this->_props) {
            if(props->getGraphicComponent()->id() == gc.id()) {
                return props;
            }
        }
        return nullptr;
    }
}