#pragma once

#include <map>

#include <engine/io/Window.hpp>

#include <kernel/front/props/Props.hpp>

#include "resolver/GraphicResolver.hpp"
#include "resolver/PhysicResolver.hpp"

namespace megu::kernel {
    class PhysicResolver;
    class GraphicResolver;

    class Kernel {
        public:
            Kernel() = delete;
            Kernel(Window &);
           ~Kernel();

            void step();
            void push(Prop &);
            void remove(Prop &);

            inline PhysicEngine & getPhysicEngine() {return this->_pEngine;}
            inline GraphicEngine & getGraphicEngine() {return this->_gEngine;}

            inline Window & window() {return this->_window;}

            Prop * get(const Prop::Physical_Component &);
            Prop * get(const Prop::Graphical_Component &);

        private:
            Window & _window;

            PhysicEngine _pEngine; 
            GraphicEngine _gEngine;

            Identifiable_Map<Prop> _props;
            PhysicResolver _pResolver;
            GraphicResolver _gResolver;
    };
}