#pragma once

#include <map>
#include <string>
#include <chrono>

#include <kernel/back/engine/GraphicEngine.hpp>
#include <kernel/back/component/Graphical.hpp>
#include <kernel/back/linker/GraphicLinker.hpp>

#include <engine/utility/ref_set.hpp>
#include <engine/graphics/front/object/Sprite.hpp>
#include <engine/graphics/front/module/Sprite_Module.hpp>

namespace megu::kernel {
    using Frame_List = std::vector<megu::Sprite::Frame>;
    using Frame = megu::Sprite::Frame;

    class Sprite : public Graphical<GraphicEngine>, public megu::Sprite {
        public:
            Sprite();
            Sprite(const std::filesystem::path &, const Frame & = {0, 0, 0, 0});
   virtual ~Sprite() = default;

            inline double getDuration() const {return this->_duration;}
            inline size_t getCurrentFrameIndex() const {return this->_index;}
            const Frame & getCurrentFrame(size_t, size_t) const;
            const Frame_List & getCurrentAnimations(size_t) const;

            inline void setDuration(double d) {this->_duration = d;}
            void setCurrentAnimation(size_t);
            
            void pushAnimation(size_t, const Frame_List & = {});
            void pushAnimation(size_t, const Frame &);
            void pushAnimation(size_t, float, float, float, float);

            void removeAnimation(size_t);
            void removeAnimation(size_t, size_t);

            void apply(GraphicEngine &) override;
            void unapply(GraphicEngine &) override;
            void update(double) override;

            static const GraphicLinker<megu::Sprite> & linker() {return Sprite::_Linker;}

        private:
            std::map<size_t, Frame_List> _frames;
            size_t _animation;
            size_t _index;
            double _duration;
            double _previous;

            static GraphicLinker<megu::Sprite> _Linker;
    };
}