#include "TileMap.hpp"

namespace megu::kernel {
    UniqueGraphicLinker<TileArray> Tilemap::_Linker = UniqueGraphicLinker<TileArray>();

    Tilemap::Tilemap(const std::filesystem::path & path, size_t width, size_t height, float size, size_t lenght, double aUpdate) 
    : TileArray(path, width , height, size, lenght), _tileSize(lenght), _duration(aUpdate) {
        /*if(lenght % width != 0 && lenght % height != 0) {
            throw std::runtime_error("Tilemap dimension not matching tiles size.");
        }*/
        
        size_t tileWidth = this->getTextureWidth() / lenght;
        size_t tileHeight = this->getTextureHeight() / lenght;

        for(size_t y = 0; y < tileHeight; ++y) {
            for(size_t x = 0; x < tileWidth; ++x) {
                this->_tilesPosition.push_back({x * lenght, y * lenght});
            }
        }
        
        size_t i = 0;
        for(size_t x = 0; x < width * lenght; x += lenght) {
            this->_tilesValue.push_back({});
            for(size_t y = 0; y < height * lenght; y += lenght) {
                this->_tilesValue[i].push_back(0);
            }
            ++i;
        }

        if(!Tilemap::_Linker.haveModule()) {
            Tilemap::_Linker.setModule(new TileArray_Module{});
        }
    }

    void Tilemap::setValue(size_t x, size_t y, size_t value) {
        if(this->row() <= x || this->collumn() <= y) {
            throw std::runtime_error("Set Tilemap coords out of range.");
        }

        if(value >= this->_tilesPosition.size()) {
            std::cerr << value << " for " << this->_tilesPosition.size() << std::endl;
            throw std::runtime_error("Set Tilemap value out of range.");
        }

        TilePosition position = this->_tilesPosition[value];
        glm::vec4 uv = {position.x, 
                        this->texture().height() - position.y - this->_tileSize, 
                        this->_tileSize, 
                        this->_tileSize};

        this->setUv(this->collumn() - 1 - y, x, uv);
        this->_tilesValue[x][y] = value;
    }

    size_t Tilemap::getValue(size_t x, size_t y) const {
        TilePosition position = {x, y};
        if(this->_animations.contains(position)) {
            return this->_animations.at(position).first.at(this->_animations.at(position).second);
        }

        if(this->_tilesValue.size() > x && this->_tilesValue[x].size() > y) {
            return this->_tilesValue[x][y];
        }
        throw std::runtime_error("Tilemap coords out of bound");
    }

    void Tilemap::pushAnimation(size_t x, size_t y, const std::vector<size_t> & ids) {
        if(x < this->row() && y < this->collumn()) {
            this->_animations[{x, y}] = std::pair<std::vector<size_t>, size_t>(ids, 0);
        }
    }

    void Tilemap::removeAnimation(size_t x, size_t y) {
        this->_animations.erase({x, y});
    }

    void Tilemap::update(double time) {
        if(this->_previous == 0.0) {
            this->_previous = time;
        }
        else {
            if(time - this->_previous > this->_duration) {
                for(auto & [coords, animation] : this->_animations) {
                    animation.second = (animation.second+1) % animation.first.size();
                    this->setValue(coords.x, coords.y, animation.first[animation.second]);
                }
                this->_previous = time;
            }
        }
    }

    void Tilemap::apply(GraphicEngine & engine) {
        Tilemap::_Linker.link(*this, this->getLayerPriority(), this->getObjectPriority());
        Tilemap::_Linker.link(engine);
    }

    void Tilemap::unapply(GraphicEngine & engine) {
        Tilemap::_Linker.unlink(*this);
        Tilemap::_Linker.link(engine);
    }
}