#pragma once

#include <map>
#include <vector>

#include <kernel/back/engine/GraphicEngine.hpp>
#include <kernel/back/component/Graphical.hpp>
#include <kernel/back/linker/UniqueLinker.hpp>

#include <engine/graphics/front/object/TileArray.hpp>
#include <engine/graphics/front/module/TileArray_Module.hpp> 

namespace megu::kernel {
    class Tilemap : public Graphical<GraphicEngine>, public megu::TileArray {
        public:
            struct TilePosition {
                size_t x = 0;
                size_t y = 0;
            };

            Tilemap() = delete;
            Tilemap(const std::filesystem::path &, size_t, size_t, float, size_t, double = 0.1);
   virtual ~Tilemap() = default;

            inline void setDuration(double duration) {this->_duration = duration;} 
            void setValue(size_t, size_t, size_t);

            inline double getDuration() const {return this->_duration;}
            size_t getValue(size_t, size_t) const;

            void pushAnimation(size_t, size_t, const std::vector<size_t> &);
            void removeAnimation(size_t, size_t);
            
            void apply(GraphicEngine &) override;
            void unapply(GraphicEngine &) override;
            void update(double) override;

        private:
            std::vector<TilePosition> _tilesPosition;
            std::vector<std::vector<size_t>> _tilesValue;
            std::map<TilePosition, std::pair<std::vector<size_t>, size_t>, reference_sorter<TilePosition>> _animations;
            TileArray_Module _module;
            size_t _tileSize;
            double _duration;
            double _previous;

            static UniqueGraphicLinker<TileArray> _Linker;
    };
}