#include "Fixed.hpp"

#include <kernel/front/Kernel.hpp>

namespace megu::kernel {
    Fixed::Fixed(float x, float y, float w, float h, CollideLambda cl, UpdateLambda ul)
    : TangibleStatic(Position(x, y), Dimension(w, h, 0.f)), _collide(cl), _update(ul) {} 

    void Fixed::updatePhysic(double time) const {
        if(this->_update != nullptr) {
            this->_update(time);
        }
    }

    void Fixed::onCollide(Kernel & kernel, PhysicEngine & engine, Identifiable & id, Physical & physical, double time) {
        if(this->_collide != nullptr) {
            this->_collide(kernel, engine, id, physical, time);
        }
    }

    void Fixed::apply(PhysicEngine & engine) {
        engine.get().push(this->getLayer(), *this);
    }

    void Fixed::unapply(PhysicEngine & engine) {
        engine.get().remove(*this);
    }

    void Fixed::setCollideLambda(CollideLambda lambda) {
        this->_collide = lambda;
    }

    void Fixed::setUpdateLambda(UpdateLambda lambda) {
        this->_update = lambda;
    }
}