#include "Movable.hpp"

namespace megu::kernel {
    Movable::Movable(float x, float y, float w, float h, CollideLambda cl, UpdateLambda ul)
    : TangibleMovable(Position(x, y), Dimension(w, h, 0.f)), _collide(cl), _update(ul) {}

    void Movable::updatePhysic(double time) {
        if(this->_update != nullptr) {
            this->_update(time);
        }
    }

    void Movable::onCollide(Kernel & kernel, PhysicEngine & engine, Identifiable & id, Physical & physical, double time) {
        if(this->_collide != nullptr) {
            this->_collide(kernel, engine, id, physical, time);
        }
    }

    void Movable::apply(PhysicEngine & engine) {
        engine.get().push(this->getLayer(), *this);
    }

    void Movable::unapply(PhysicEngine & engine) {
        engine.get().remove(*this);
    }

    void Movable::setCollideLambda(CollideLambda lambda) {
        this->_collide = lambda;
    }

    void Movable::setUpdateLambda(UpdateLambda lambda) {
        this->_update = lambda;
    }
}