#pragma once

#include <kernel/back/component/Physical.hpp>
#include <kernel/back/engine/PhysicEngine.hpp>
#include <vector>
#include <functional>

#include "Tile.hpp"

namespace megu::kernel {
    class TileArray : public Physical<PhysicEngine>, public TangibleStatic {
        public:
            TileArray() = delete;
            TileArray(float, float, float, float, CollideLambda = nullptr, UpdateLambda = nullptr);
   virtual ~TileArray() = default;

            std::optional<std::reference_wrapper<const Tile>> at(const Position &) const;

            void push(const Position &, Tile &);
            void remove(const Tile &);

            using TileCollideLambda = std::function<void(Kernel &, PhysicEngine &, Identifiable &, Tile &, double)>;
        
            void setCollideLambda(TileCollideLambda);
            void setUpdateLambda(UpdateLambda);

            void updatePhysic(double) const override;
            void apply(PhysicEngine &) override;
            void unapply(PhysicEngine &) override;
            void onCollide(Kernel &, PhysicEngine &, Identifiable &, Physical &, double) override;

            virtual std::optional<SquareBox> isColliding(const SquareBox &) const override;

        private:
            std::map<Position, std::reference_wrapper<Tile>, reference_sorter<Position>> _tiles;
            TileCollideLambda _collide;
            UpdateLambda _update;
    };
}